#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>

// This is a trivial program which you kick off from the linux command-line
// and it waits until it gets input from the serial line /dev/ttyUSB0 at
// which point it announces itself and issues a prompt and accepts trivial
// commands.

#define BUF_SIZE 512
static char local_buf[BUF_SIZE], remote_buf[BUF_SIZE];

#define BIG_BUF_SIZE 0x10000
static char big_local_buf[BIG_BUF_SIZE], big_remote_buf[BIG_BUF_SIZE];

fd_set inFds;

int Data_From_USB_Fd, Data_To_USB_Fd, Data_From_KBD_Fd, Data_To_VDU_Fd;

#ifndef FALSE
#define FALSE (0!=0)
#define TRUE (0==0)
#endif

char *progname;

int debug = FALSE;

FILE *sourcefile;
char *curfile;
int startline = TRUE, lineno = 1, col = 0, ch;

int main (int argc, char **argv)
{
  struct termios ttyOrig, usbOrig;
  char ticker, tmp;
  int retVal;
  int serial_port;
  size_t kbd_bytes_got, remote_bytes_got, usb_writes_remaining, vdu_writes_remaining;
  char *s;

  progname = argv[0];
  if ((s = strrchr (progname, '/')) != NULL)
    progname = s + 1;		// Unix
  if ((s = strrchr (progname, '\\')) != NULL)
    progname = s + 1;		// M$
  if ((s = strrchr (progname, ']')) != NULL)
    progname = s + 1;		// Dec
  if ((s = strrchr (progname, ';')) != NULL)
    *s = '\0';			// Version no's
  if (((s = strrchr (progname, '.')) != NULL)
      && (strcasecmp (s, ".exe") == 0))
    *s = '\0';
  if (((s = strrchr (progname, '.')) != NULL)
      && (strcasecmp (s, ".com") == 0))
    *s = '\0';

  serial_port = 1;
  while ((argc > 1) && (*argv[1] == '-')) {
    if ((argc > 1) && (strcmp (argv[1], "-0") == 0)) {
      argv++;
      argc--;
      serial_port = 0;
    }
    if ((argc > 1) && (strcmp (argv[1], "-d") == 0)) {
      argv++;
      argc--;
      debug = TRUE;
    }
    if ((argc > 1) && (*argv[1] == '-') && (strcmp (argv[1], "-0")) && (strcmp (argv[1], "-d"))) {
      fprintf (stderr, "syntax: %s [-d] [-0]\n", progname);
      exit (1);
    }
  }

  // This sets our USB Serial line into the ACIA-compatible mode
  if (serial_port == 0) {
    system("stty -F /dev/ttyUSB0 speed 115200 pass8 cstopb -brkint -icrnl -imaxbel -opost -onlcr -isig -icanon -iexten -echo -echoe -echok -echoctl -echoke > /dev/null");
    //system("stty < /dev/ttyUSB0");
  } else {
    system("stty -F /dev/ttyUSB1 speed 115200 pass8 cstopb -brkint -icrnl -imaxbel -opost -onlcr -isig -icanon -iexten -echo -echoe -echok -echoctl -echoke > /dev/null");
    //system("stty < /dev/ttyUSB1");
  }

  Data_From_USB_Fd = Data_To_USB_Fd = open ((serial_port == 0 ? "/dev/ttyUSB0" : "/dev/ttyUSB1"), O_RDWR);
  if (Data_From_USB_Fd < 0) {
    fprintf(stderr, "Failure: open - %s\n", strerror(errno)); exit(EXIT_FAILURE);
  } else {
    int flags = fcntl(Data_From_USB_Fd, F_GETFL, 0);
    fcntl(Data_From_USB_Fd, F_SETFL, flags | O_NONBLOCK);
  }
  
  //tcsendbreak (Data_To_USB_Fd, 0);

  Data_From_KBD_Fd = STDIN_FILENO;
  {
    int flags = fcntl(Data_From_KBD_Fd, F_GETFL, 0);
    fcntl(Data_From_KBD_Fd, F_SETFL, flags | O_NONBLOCK);
  }
  Data_To_VDU_Fd = STDOUT_FILENO;

  // Put tty into single character raw mode
  if (tcgetattr (Data_From_KBD_Fd, &ttyOrig) == -1) {
    fprintf(stderr, "Failure: tcgetattr - %s\n", strerror(errno)); exit(EXIT_FAILURE);
  } else {
    struct termios t;

    //if (tcgetattr (STDIN_FILENO, &t) == -1)     /* return -1 */ ;
    //ttyOrig = t;
    
    t = ttyOrig;
    
    t.c_lflag &= ~(ICANON | ISIG | IEXTEN | ECHO);
    /* Noncanonical mode, disable signals, extended input processing, and
       echoing */

    t.c_iflag &=
     ~(BRKINT | ICRNL | IGNBRK | IGNCR | INLCR | INPCK | ISTRIP | IXON |
       PARMRK);
    /* Disable special handling of CR, NL, and BREAK. No 8th-bit stripping or
       parity error handling. Disable START/STOP output flow control. */

    t.c_oflag &= ~OPOST;        /* Disable all output processing */

    t.c_cc[VMIN] = 1;           /* Character-at-a-time input */
    t.c_cc[VTIME] = 0;          /* with blocking */

    if (tcsetattr (Data_From_KBD_Fd, TCSAFLUSH, &t) == -1)  /* return -1 */
      ;

  }

  // Put USB into single character raw mode
  if (tcgetattr (Data_From_USB_Fd, &usbOrig) == -1) {
    fprintf(stderr, "Failure: tcgetattr - %s\n", strerror(errno)); exit(EXIT_FAILURE);
  } else {
    struct termios t;

    //if (tcgetattr (STDIN_FILENO, &t) == -1)     /* return -1 */ ;
    //ttyOrig = t;
    
    t = usbOrig;
    
    t.c_lflag &= ~(ICANON | ISIG | IEXTEN | ECHO);
    /* Noncanonical mode, disable signals, extended input processing, and
       echoing */

    t.c_iflag &=
     ~(BRKINT | ICRNL | IGNBRK | IGNCR | INLCR | INPCK | ISTRIP | IXON |
       PARMRK);
    /* Disable special handling of CR, NL, and BREAK. No 8th-bit stripping or
       parity error handling. Disable START/STOP output flow control. */

    t.c_oflag &= ~OPOST;        /* Disable all output processing */

    t.c_cc[VMIN] = 1;           /* Character-at-a-time input */
    t.c_cc[VTIME] = 0;          /* with blocking */

    if (tcsetattr (Data_From_USB_Fd, TCSAFLUSH, &t) == -1)  /* return -1 */
      ;

  }

  fprintf(stderr, "Connected to 6809 over /dev/ttyUSB%c.  Type ^D to exit.\r\n", serial_port+'0');
  if (serial_port == 0) {
    //system("stty < /dev/ttyUSB0");
  } else {
    //system("stty < /dev/ttyUSB1");
  }

  // Terminal emulation
  usb_writes_remaining = 0; vdu_writes_remaining = 0;
  ticker = ' '+1;
  struct timeval zero = { 0, 0 };
  for (;;) {

    // write(2, &ticker, 1); tmp = 8; write(2, &tmp, 1);  ticker += 1; if (ticker == '~') ticker = ' '+1; // show if main loop is alive

    FD_ZERO (&inFds); // move these out the loop as soon as tested OK.
    FD_SET (Data_From_KBD_Fd, &inFds);
    FD_SET (Data_From_USB_Fd, &inFds);

    retVal = select((Data_From_USB_Fd > Data_From_KBD_Fd ? Data_From_USB_Fd+1 : Data_From_KBD_Fd+1), &inFds, NULL, NULL, &zero);
    
    if (retVal < 0) {
      fprintf(stderr, "Failure: select - %s\n", strerror(errno));
      (void) tcsetattr (Data_From_KBD_Fd, TCSANOW, &ttyOrig);   // restore keyboard/stdin
      exit(EXIT_FAILURE);
    }

    if (FD_ISSET (Data_From_KBD_Fd, &inFds)) { // Data to send
      if (debug) fprintf(stderr, "From Kbd: data available\r\n");
      kbd_bytes_got = read (Data_From_KBD_Fd, local_buf, BUF_SIZE);
      if (kbd_bytes_got <= 0) exit (EXIT_SUCCESS); // should never happen in raw mode
      if (debug) fprintf(stderr, "From Kbd: %d\r\n", kbd_bytes_got);
      {int i;
	for (i = 0; i < kbd_bytes_got; i++) {
	  if (local_buf[i] == ('D'&31)) {
            (void) tcsetattr (Data_From_KBD_Fd, TCSANOW, &ttyOrig);   // restore keyboard/stdin
	    fprintf(stderr, "\rTerminal exiting.\r\n");
            exit(EXIT_SUCCESS);
	  }
	}
      }
      // this isn't very efficient but handling a cyclic buffer is too complex to add right now:
      if (debug) fprintf(stderr, "memmove(%p,%p,%d)\r\n", big_local_buf+usb_writes_remaining, local_buf, kbd_bytes_got);
      memmove(big_local_buf+usb_writes_remaining, local_buf, kbd_bytes_got);
      usb_writes_remaining += kbd_bytes_got;
      //continue;
    }

    if (FD_ISSET (Data_From_USB_Fd, &inFds)) { // Data received
      if (debug) fprintf(stderr, "From USB: data available\r\n");
      remote_bytes_got = read (Data_From_USB_Fd, remote_buf, BUF_SIZE);
      if (remote_bytes_got <= 0) exit (EXIT_SUCCESS);
      if (debug) fprintf(stderr, "From USB: %d\r\n", remote_bytes_got);
      if (debug) fprintf(stderr, "memmove(%p,%p,%d)\r\n", big_remote_buf+vdu_writes_remaining, remote_buf, remote_bytes_got);
      memmove(big_remote_buf+vdu_writes_remaining, remote_buf, remote_bytes_got);
      vdu_writes_remaining += remote_bytes_got;
      //continue;
    }

    if (usb_writes_remaining > 0) {
      if (debug) fprintf(stderr, "To USB: pending %d\r\n", usb_writes_remaining);
      int usb_bytes_written = write(Data_To_USB_Fd, local_buf, usb_writes_remaining);
      if (debug) fprintf(stderr, "To USB: %d\r\n", usb_bytes_written);
      // This memmove must handle overlaps. So don't use memcpy.
      if (debug) fprintf(stderr, "memmove(%p,%p,%d)\r\n", local_buf, local_buf+usb_bytes_written, usb_writes_remaining-usb_bytes_written);
      memmove(local_buf, local_buf+usb_bytes_written, usb_writes_remaining-usb_bytes_written);
      usb_writes_remaining -= usb_bytes_written;
      //continue;
    }

    if (vdu_writes_remaining > 0) {
      if (debug) fprintf(stderr, "To VDU: pending %d\r\n", vdu_writes_remaining);
      int vdu_bytes_written = write(Data_To_VDU_Fd, remote_buf, vdu_writes_remaining);
      if (debug) fprintf(stderr, "To VDU: wrote %d\r\n", vdu_bytes_written);
      if (debug) fprintf(stderr, "memmove(%p,%p,%d)\r\n", remote_buf, remote_buf+vdu_bytes_written, vdu_writes_remaining-vdu_bytes_written);
      memmove(remote_buf, remote_buf+vdu_bytes_written, vdu_writes_remaining-vdu_bytes_written);
      vdu_writes_remaining -= vdu_bytes_written;
      continue;
    }

    
    //usleep(10);

  }

  (void) tcsetattr (Data_From_KBD_Fd, TCSANOW, &ttyOrig);   // restore keyboard/stdin
  return (EXIT_FAILURE);

}
