        ;;      Title:          Initialize I/O Ports
        ;;      Name:           IPORTS
        ;;
        ;;      Purpose:        Initialize I/O ports from an array of port addresses and values.
        ;;
        ;;      Entry:          Register X = Base address of array
        ;;
        ;;                      The array consists of 3 byte elements
        ;;                      array+0 = High byte of port 1 address
        ;;                      array+1 = Low  byte of port 1 address
        ;;                      array+2 = Value to store in port 1 address
        ;;                      array+3 = High byte of port 2 address
        ;;                      array+4 = Low  byte of port 2 address
        ;;                      array+5 = Value to store in port 2 address
        ;;                              .
        ;;                              .
        ;;                              .
        ;;
        ;;      Exit:   None Registers Used: A,B,CC,U,X
        ;;
        ;;      Time:   10 cycles overhead plus 23 * N cycles for each port,
        ;;              where N is the number of bytes.
        ;;
        ;;      Size:   Program 13 bytes
        ;;
IPORTS:
        ;;
        ;;  EXIT IMMEDIATELY IF NUMBER OF PORTS IS ZERO
        ;;
        TSTA                    ; TEST NUMBER OF PORTS
        BEQ     EXITIP          ; BRANCH IF NO PORTS TO INITIALIZE
        ;;
        ;;  LOOP INITIALIZING PORTS
        ;;
INITPT:
        LDU     ,X++            ; GET NEXT PORT ADDRESS
        LDB     ,X+             ; GET VALUE TO SEND THERE
        STB     ,U              ; SEND VALUE TO PORT ADDRESS
        DECA                    ; COUNT PORTS
        BNE     INITPT          ; CONTINUE UNTIL ALL PORTS INITIALIZED
        ;;
        ;;  EXIT
        ;;
EXITIP:
        RTS

PINIT:
; 
; INITIALIZE 6820 OR 6821 PERIPHERAL INTERFACE ADAPTER (PIA)
;
	;	PORT A = INPUT
	;	CA1 =	DATA AVAILABLE, SET ON LOW TO HIGH TRANSITION,
	;		NO INTERRUPTS
	;	CA2 =	DATA ACKNOWLEDGE HANDSHAKE
	;
	FDB	PIACRA		; PIA CONTROL REGISTER A ADDRESS
	FDB	00000000B	; INDICATE NEXT ACCESS TO DATA
				; DIRECTION REGISTER (SAME ADDRESS
				; AS DATA REGISTER) 

	FDB	PIADRA		; PIA DATADIRECTION REGISTER A ADDRESS
	FDB	00000000B	; ALL BITS INPUT

	FDB	PIACRA		; PIA CONTROL REGISTER A ADDRESS
	FDB	00100110B	; BITS 7,6 NOT USED
				; BIT 5 = 1 TO MAKE CA2 OUTPUT
				; BIT 4 = 0 TO MAKE CA2 A PULSE
				; BIT 3 = 0 TO MAKE CA2 INDICATE
				; 			DATA REGISTER FULL 
				; BIT 2 = 1 TO ADDRESS DATA REGISTER
				; BIT 1 = 1 TO MAKE CA1 ACTIVE
				; 			LOW TO HIGH
				; BIT 0 = 0 TO DISABLE CA1 INTERRUPTS 
	;
	;	PORT B = OUTPUT
	;	CB1 =	DATA ACKNOWLEDGE, SET ON HIGH TO LOW TRANSITION,
	;		NO INTERRUPTS
	;	CB2 =	DATA AVAILABLE, CLEARED BY WRITING TO DATA
	;		REGISTER B, SET TO 1 BY HIGH TO LOW TRANSITION ON CB1
	;
	FDB	PIACRB		; PIA CONTROL REGISTER B ADDRESS 
	FDB	00000000B	; INDICATE NEXT ACCESS TO DATA
				; DIRECTION REGISTER (SAME ADDRESS
				; AS DATA REGISTER 
	FDB	PIADRB		; PIA DATA DIRECTION REGISTER B
	FDB	11111111B	; ADDRESS ALL BITS OUTPUT
	FDB	PIACRB		; PIA CONTROL REGISTER B ADDRESS
	FDB	00100100B	; BITS 7,6 NOT USED
				; BIT 5 = 1 TO MAKE CB2 OUTPUT
				; BIT 4 = 0 TO MAKE CB2 A PULSE
				; BIT 3 = 0 TO MAKE CB2 INDICATE DATA REGISTER FULL 
				; BIT 2 = 1 TO ADDRESS DATA REGISTER
				; BIT 1 = 0 TO MAKE CB2 ACTIVE
				; BIT 0 = 0 TO DISABLE CB1 INTERRUPTS
