        ;;      Title:          Initialize I/O Ports
        ;;      Name:           IPORTS
        ;;
        ;;      Purpose:        Initialize I/O ports from an array of port addresses and values.
        ;;
        ;;      Entry:          Register X = Base address of array
        ;;
        ;;                      The array consists of 3 byte elements
        ;;                      array+0 = High byte of port 1 address
        ;;                      array+1 = Low  byte of port 1 address
        ;;                      array+2 = Value to store in port 1 address
        ;;                      array+3 = High byte of port 2 address
        ;;                      array+4 = Low  byte of port 2 address
        ;;                      array+5 = Value to store in port 2 address
        ;;                              .
        ;;                              .
        ;;                              .
        ;;
        ;;      Exit:   None Registers Used: A,B,CC,U,X
        ;;
        ;;      Time:   10 cycles overhead plus 23 * N cycles for each port,
        ;;              where N is the number of bytes.
        ;;
        ;;      Size:   Program 13 bytes
        ;;
IPORTS:
        ;;
        ;;  EXIT IMMEDIATELY IF NUMBER OF PORTS IS ZERO
        ;;
        TSTA                    ; TEST NUMBER OF PORTS
        BEQ     EXITIP          ; BRANCH IF NO PORTS TO INITIALIZE
        ;;
        ;;  LOOP INITIALIZING PORTS
        ;;
INITPT:
        LDU     ,X++            ; GET NEXT PORT ADDRESS
        LDB     ,X+             ; GET VALUE TO SEND THERE
        STB     ,U              ; SEND VALUE TO PORT ADDRESS
        DECA                    ; COUNT PORTS
        BNE     INITPT          ; CONTINUE UNTIL ALL PORTS INITIALIZED
        ;;
        ;;  EXIT
        ;;
EXITIP:
        RTS

                                ;
; INITIALIZE 6850 ASYNCHRONOUS COMMUNICATIONS INTERFACE ADAPTER
; (ACIA OR UART)
;
	; 8 BIT DATA, NO PARITY
	; 1 STOP BIT
	; DIVIDE MASTER CLOCK BY 16
	; NO INTERRUPTS
	;
	FDB 	ACIACR		; ACIA CONTROL REGISTER ADDRESS 
	FCB 	00000011B	; PERFORM MASTER RESET
				; 6850 HAS NO RESET INPUT 
	FDB	ACIACR		; ACIA CONTROL REGISTER ADDRESS 
	FCB	00010101B	; BIT 7 = 0 TO DISABLE RECEIVE INTERRUPTS 
				; BIT 6 = 0 TO MAKE RTS LOW 
				; BIT 5 = 0 TO DISABLE TRANSMIT INTERRUPTS
				; BIT 4 = 1 TO SELECT DATA
				; BIT 3 = 0 FOR NO PARITY
				; BIT 2 = 1 FOR 1 STOP BIT
				; BIT 1 = 0, BIT 0 = 1 TO DIVIDE MASTER CLOCK BY 16
