#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include "vecx.h"

int main(int argc, char **argv)
{
  int F[0x10000];
  FILE *inf, *orig_dat, *dat;
  uint32_t Flags;
  uint16_t Base, Index, DP;
  uint16_t Version;
  int dup;
  int addr, i, rc;
  char CartName[512];
  
  if (argc != 4) {
    fprintf(stderr, "syntax: inf2dat file.inf file.dat file.bin.dat\n");
    exit(1);
  }

  if (strcmp(argv[2], argv[3]) == 0) {
    fprintf(stderr, "inf2dat: The old and new dat files appear to be the same: %s\n", argv[2]);
    exit(1);
  }

  inf = fopen(argv[1], "r");
  if (inf == NULL) {
    fprintf(stderr, "inf2dat: cannot open input file %s\n", argv[1]);
    exit(1);
  }
  orig_dat = fopen(argv[2], "r");
  if (orig_dat == NULL) {
    fprintf(stderr, "inf2dat: cannot open original dat file %s\n", argv[2]);
    exit(1);
  }

  for (i = 0; i <= 0xFFFF; i++) F[i] = 0;

  dat = fopen(argv[3], "w");
  if (dat == NULL) {
    fprintf(stderr, "inf2dat: cannot open output file %s\n", argv[3]);
    exit(1);
  }

  for (;;) {
    int rc;
    char line[1025]; line[1024] = '\0';
    char *s = line;
    int len = 1024;
    unsigned short Address;
    rc = getline(&s, &len, orig_dat);
    if ((rc < 0) || (s == NULL)) break;
    rc = sscanf(line, "code %hX", &Address);
    if (rc == 1) {
      F[Address] |= F_OPCODE;
    } else {
      fprintf(dat, "%s", line);
    }
  }

  

  rc = fscanf(inf, "V %hX C %s", &Version, CartName);
  if (rc != 2) {
    fprintf(stderr, "Bad header - scanned %d items\n", rc);
    exit(1);
  }

  i = 0;
  while (i <= 0xFFFF) {
    rc = fscanf(inf, "%X F %X D %hX B %hX I %hX %d", &addr, &Flags, &DP, &Base, &Index, &dup);
    if (feof(inf)) {
      fprintf(stderr, "End of data at address %4X\n", addr);
      exit(1);
    }
    if (rc != 6) {
      fprintf(stderr, "Bad data - scanned %d items\n", rc);
      exit(1);
    }
    if (addr != i) {
      fprintf(stderr, "Address mismatch at %04X != %04X\n", addr, i);
      exit(1);
    }
    for (;;) {
      if ((F[i]|Flags)&F_OPCODE) {
        fprintf(dat, "code %04X\n", i);
      }
      if (dup == 0) break;
      if (i >= 0x8000) break; // of 0xFFFF for end of address space
      dup -= 1; i += 1;
    }
    if (i >= 0x8000) break; // again  use 0xFFFF for complete memory
    i += 1;
  }

  
  exit(0);
  return 0;
}
