#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <error.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>

static char errmess[1024];

int poll_input(void) {
  char input[1024];
  int rc;
  rc = read(0 /* stdin */, input, 1);
  if (rc == 0) {
    strcpy(errmess, "EOF");
    return rc;
  } else if (rc < 0) {
    if (errno == EAGAIN) {
      sprintf(errmess, "rc=%d errno=EAGAIN", rc);
    } else if (errno == EWOULDBLOCK) {
      sprintf(errmess, "rc=%d errno=EWOULDBLOCK", rc);
    } else if (errno == EBADF) {
      sprintf(errmess, "rc=%d errno=EBADF", rc);
    } else if (errno == EFAULT) {
      sprintf(errmess, "rc=%d errno=EFAULT", rc);
    } else if (errno == EINTR) {
      sprintf(errmess, "rc=%d errno=EINTR", rc);
    } else if (errno == EINVAL) {
      sprintf(errmess, "rc=%d errno=EINVAL", rc);
    } else if (errno == EIO) {
      sprintf(errmess, "rc=%d errno=EIO", rc);
    } else if (errno == EISDIR) {
      sprintf(errmess, "rc=%d errno=EISDIR", rc);
    } else {
      sprintf(errmess, "rc=%d strerror=\"%s\" errno=%d", rc, strerror(errno), errno);
    }
    return rc;
  }
  errmess[0] = '\0';
  return input[0];
}

int main(int argc, char **argv) {
  int c, lp = 0, seq = 0;
  char line[1024], report[2048];
  int flags = fcntl(0, F_GETFL, 0);
  int written, bytes_to_write;
  fcntl(0 /* stdin */, F_SETFL, flags | O_NONBLOCK);
  report[0] = errmess[0] = line[0] = '\0';
  for (;;) {
    c = poll_input();
    if (c == 0) {
      usleep(1000);
      fprintf(stderr, "EOF. '%s'  lasterr='%s'\n", line, errmess);
      exit(0);
    } else if (c > 0) {
      if (c < 32) {
        line [lp++] = '^'; line [lp++] = c+'@'; line[lp] = '\0';
      } else {
        line [lp++] = c; line[lp] = '\0';
      }
      sprintf(report, "got: '%s'  lasterr='%s'\n", line, errmess);
    } else { // c < 0 - error.
      // EAGAIN
      sprintf(report, "ERROR: '%s'  lasterr='%s'  seq=%d\n", line, errmess, ++seq);
    }
    if (c >= 0 || errno != EAGAIN) {
      bytes_to_write = strlen(report);
      written = 0;
      while (written < bytes_to_write) {
        int delta;
        delta = write(2 /* stderr */, report+written, bytes_to_write-written);
        if (delta == 0) {
          usleep(1000);
        } else if (delta < 0) {
          usleep(1000);
          if (errno != 11) {
            fprintf(stderr, "Write fails - %s (Error %d)\n", strerror(errno), errno);
            exit(errno);
          }
        }
        written += delta;
      }
    }
    if (lp == 1023) {
      lp = 0; report[0] = errmess[0] = line[0] = '\0';
    }
  }
  exit(0);
  return 0;
}
