#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <error.h>
#include <errno.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <libgen.h>
#include <termios.h>
#include <ncurses.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdarg.h>


#define BUF_SIZE 500
static char remote_buf[BUF_SIZE];

fd_set inFds;

int Data_From_USB_Fd, Data_To_USB_Fd;

#ifndef FALSE
#define FALSE (0!=0)
#define TRUE (0==0)
#endif

char *progname;

int debug = FALSE;

FILE *sourcefile;
char *curfile;
int startline = TRUE, lineno = 1, col = 0, ch;

extern int gettimeofday(struct timeval *tv, void *tz);

#ifdef Vectrex_SDL
#include <SDL.h>
#include <SDL_image.h>
#endif
//#include "e6522.h"
#include "e6809.h"
//#include "e8910.h"
#include "edac.h"
#include "ser.h"
#include "vecx.h"

#ifndef Uint32
#define Uint32 uint32_t
#endif

#ifndef Uint64
#define Uint64 uint64_t
#endif

enum {
  EMU_TIMER = 20, /* the emulators heart beats at 20 milliseconds */

  DEFAULT_WIDTH = 495,
  DEFAULT_HEIGHT = 615
};

vecx vectrex;
#ifdef Vectrex_SDL
static SDL_Window *window = NULL;
static SDL_Renderer *renderer = NULL;
static SDL_Texture *overlay = NULL;
static SDL_Texture *buffer = NULL;
static SDL_Texture *buffer2 = NULL;

static int32_t scl_factor;
#endif

static void quit(void);

/* command line arguments */
static char *bios_filename = "bios.bin";
static char *cart_filename = NULL;
static char *overlay_filename = NULL;
static char fullscreen = 0;

static void render(void) {
#ifdef Vectrex_SDL
  SDL_SetRenderTarget(renderer, buffer);
  {
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 128);
    SDL_RenderFillRect(renderer, NULL);

    for (size_t v = 0; v < vectrex.vector_draw_cnt; v++) {
      Uint8 c = vectrex.vectors[v].color * 256 / VECTREX_COLORS;
      int x0 = vectrex.vectors[v].x0 / scl_factor;
      int y0 = vectrex.vectors[v].y0 / scl_factor;
      int x1 = vectrex.vectors[v].x1 / scl_factor;
      int y1 = vectrex.vectors[v].y1 / scl_factor;

      SDL_SetRenderDrawColor(renderer, 255, 255, 255, c);
      if (x0 == x1 && y0 == y1) {
        /* point */
        SDL_RenderDrawPoint(renderer, x0, y0);
        SDL_RenderDrawPoint(renderer, x0 + 1, y0);
        SDL_RenderDrawPoint(renderer, x0, y0 + 1);
        SDL_RenderDrawPoint(renderer, x0 + 1, y0 + 1);
      } else {
        SDL_RenderDrawLine(renderer, x0, y0, x1, y1);
        SDL_RenderDrawLine(renderer, x0 + 1, y0 + 1, x1 + 1, y1 + 1);
      }
    }
  }

  SDL_SetRenderTarget(renderer, buffer2);
  { SDL_RenderCopy(renderer, buffer, NULL, NULL); }

  SDL_SetRenderTarget(renderer, NULL);
  {
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
    SDL_RenderClear(renderer);

    SDL_RenderCopy(renderer, buffer, NULL, NULL);
    SDL_RenderCopy(renderer, buffer2, NULL, NULL);

    if (overlay) {
      SDL_RenderCopy(renderer, overlay, NULL, NULL);
    }
  }
  SDL_RenderPresent(renderer);
#endif
}

#ifdef NEVER
static void load_bios(void) {
  FILE *f;
  if (!(f = fopen(bios_filename, "rb"))) {
    perror(bios_filename);
    quit();
  }
  if (fread(vectrex.rom, 1, 8192 /*sizeof(vectrex.rom)*/, f) !=
      8192 /*sizeof(vectrex.rom)*/) {
    fprintf(stderr, "Invalid bios length\n");
    quit();
  }
  fclose(f);
}

static void load_cart(void) {
  // memset(vectrex.cart, 0, sizeof(vectrex.cart));
  if (cart_filename) {
    FILE *f;
    if (!(f = fopen(cart_filename, "rb"))) {
      perror(cart_filename);
      return;
    }
    (void)fread(vectrex.cart, 1, 32768 /*sizeof(vectrex.cart)*/, f);
    fclose(f);
  }
}
#endif

static void resize(void) {
#ifdef Vectrex_SDL
  int sclx, scly;
  int screenx, screeny;
  int width, height;

  SDL_GetWindowSize(window, &screenx, &screeny);

  sclx = DAC_MAX_X / screenx;
  scly = DAC_MAX_Y / screeny;

  scl_factor = sclx > scly ? sclx : scly;
  width = DAC_MAX_X / scl_factor;
  height = DAC_MAX_Y / scl_factor;

  SDL_RenderSetLogicalSize(renderer, width, height);

  SDL_DestroyTexture(buffer);
  SDL_DestroyTexture(buffer2);

  buffer = SDL_CreateTexture(renderer, SDL_PIXELFORMAT_RGBA8888,
                             SDL_TEXTUREACCESS_TARGET, width, height);
  buffer2 = SDL_CreateTexture(renderer, SDL_PIXELFORMAT_RGBA8888,
                              SDL_TEXTUREACCESS_TARGET, width / 2, height / 2);

  SDL_SetTextureBlendMode(buffer2, SDL_BLENDMODE_BLEND);
  SDL_SetTextureAlphaMod(buffer2, 128);
#endif
}

static int readevents(void) {
#ifdef Vectrex_SDL
  SDL_Event e;
  while (SDL_PollEvent(&e)) {
    switch (e.type) {
      case SDL_QUIT:
        return 1;
        break;
      case SDL_WINDOWEVENT:
        if (e.window.event == SDL_WINDOWEVENT_RESIZED) resize();
        break;
      case SDL_DROPFILE:
        cart_filename = e.drop.file;
        load_cart();
        vecx_reset(&vectrex);
        break;
      case SDL_KEYDOWN:
        switch (e.key.keysym.sym) {
          case SDLK_ESCAPE:
            return 1;
          case SDLK_a:
            vecx_input(&vectrex, VECTREX_PAD1_BUTTON1, 1);
            break;
          case SDLK_s:
            vecx_input(&vectrex, VECTREX_PAD1_BUTTON2, 1);
            break;
          case SDLK_d:
            vecx_input(&vectrex, VECTREX_PAD1_BUTTON3, 1);
            break;
          case SDLK_f:
            vecx_input(&vectrex, VECTREX_PAD1_BUTTON4, 1);
            break;
          case SDLK_LEFT:
            vecx_input(&vectrex, VECTREX_PAD1_X, 0x00);
            break;
          case SDLK_RIGHT:
            vecx_input(&vectrex, VECTREX_PAD1_X, 0xff);
            break;
          case SDLK_UP:
            vecx_input(&vectrex, VECTREX_PAD1_Y, 0xff);
            break;
          case SDLK_DOWN:
            vecx_input(&vectrex, VECTREX_PAD1_Y, 0x00);
            break;
        }
        break;
      case SDL_KEYUP:
        switch (e.key.keysym.sym) {
          case SDLK_F1:
            vecx_load(&vectrex, "q.save");
            break;
          case SDLK_F2:
            vecx_save(&vectrex, "q.save");
            break;
          case SDLK_r:
            load_cart();
            vecx_reset(&vectrex);
            break;

          case SDLK_a:
            vecx_input(&vectrex, VECTREX_PAD1_BUTTON1, 0);
            break;
          case SDLK_s:
            vecx_input(&vectrex, VECTREX_PAD1_BUTTON2, 0);
            break;
          case SDLK_d:
            vecx_input(&vectrex, VECTREX_PAD1_BUTTON3, 0);
            break;
          case SDLK_f:
            vecx_input(&vectrex, VECTREX_PAD1_BUTTON4, 0);
            break;
          case SDLK_LEFT:
            vecx_input(&vectrex, VECTREX_PAD1_X, 0x80);
            break;
          case SDLK_RIGHT:
            vecx_input(&vectrex, VECTREX_PAD1_X, 0x80);
            break;
          case SDLK_UP:
            vecx_input(&vectrex, VECTREX_PAD1_Y, 0x80);
            break;
          case SDLK_DOWN:
            vecx_input(&vectrex, VECTREX_PAD1_Y, 0x80);
            break;
        }
        break;
    }
  }
#endif
  return 0;
}

Uint64 usecs(void) {
#ifdef Vectrex_SDL
  return SDL_GetTicks()*1000; // milliseconds * 1000 = microseconds
#else
  struct timeval tv;
  gettimeofday(&tv, NULL);
  // fprintf(stderr, "seconds: %ld\nmicro seconds: %ld\n", tv.tv_sec, tv.tv_usec);
  return tv.tv_sec * 1000000 + tv.tv_usec; // | (tv.tv_usec % 1000000);
#endif
}

void milliwait(Uint32 msecs) {
#ifdef Vectrex_SDL
  SDL_Delay(msecs);
#else
  usleep(msecs * 1000U);
#endif
}

#ifdef NEVER
static void emuloop(void) {
  Uint64 next_time = usecs() + EMU_TIMER;

  vecx_reset(&vectrex);
  for (;;) {
    vecx_emu(&vectrex, (VECTREX_MHZ / 1000) * EMU_TIMER);
    if (readevents()) break;
    {
      Uint64 now = usecs();
      if (now < next_time) {
        milliwait((next_time - now)/1000);
      } else
        next_time = now;
      next_time += EMU_TIMER;
    }
  }
}
#endif

static void load_overlay() {
#ifdef Vectrex_SDL
  if (overlay_filename) {
    SDL_Texture *image = IMG_LoadTexture(renderer, overlay_filename);
    if (image) {
      overlay = image;
      SDL_SetTextureBlendMode(image, SDL_BLENDMODE_BLEND);
      SDL_SetTextureAlphaMod(image, 128);
    } else {
      fprintf(stderr, "IMG_Load: %s\n", IMG_GetError());
    }
  }
#endif
}

static int init(void) {
#ifdef Vectrex_SDL
  if (SDL_Init(SDL_INIT_VIDEO /* | SDL_INIT_AUDIO */) < 0)  // turn off sound while developing
  {
    fprintf(stderr, "Failed to initialize SDL: %s\n", SDL_GetError());
    return 0;
  }

  window =
      SDL_CreateWindow("Vecx", SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
                       DEFAULT_WIDTH, DEFAULT_HEIGHT, SDL_WINDOW_RESIZABLE);
  if (!window) {
    fprintf(stderr, "Failed to create window: %s\n", SDL_GetError());
    return 0;
  }

  renderer = SDL_CreateRenderer(
      window, -1, SDL_RENDERER_ACCELERATED | SDL_RENDERER_TARGETTEXTURE);
  if (!renderer) {
    fprintf(stderr, "Failed to create renderer: %s\n", SDL_GetError());
    return 0;
  }

  SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_BLEND);
  SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "1");

  if (fullscreen)
    SDL_SetWindowFullscreen(window, SDL_WINDOW_FULLSCREEN_DESKTOP);
#endif
  return 1;
}

static void quit(void) {
#ifdef Vectrex_SDL
  // Sometimes the program appears to hang when you press ESCape ... still to be
  // debugged.
  //  fprintf(stderr, "Quit #1\n");
  if (renderer) SDL_DestroyRenderer(renderer);
  //  fprintf(stderr, "Quit #2\n");
  if (window) SDL_DestroyWindow(window);
  //  fprintf(stderr, "Quit #3\n");
  SDL_Quit();
  //  fprintf(stderr, "Quit #4\n");
#endif

#ifdef NEVER
  // Write the disassembly file
  // Still TO DO: Initialise these arrays by loading the same file at startup,
  // so that the file accumulates more data over time, rather than just
  // reflecting the most recent run.
  {
    int i, dup;
    FILE *dat;
    char *datfile = malloc(strlen(cart_filename) + 16);
    sprintf(datfile, "%s.inf", cart_filename);
    dat = fopen(datfile, "w");
    //  fprintf(stderr, "Quit #5\n");
    if (dat == NULL) {
      int fd = mkstemp("/tmp/tmpXXXXXX");
      // sprintf(datfile, "%s", tmpnam(NULL));
      dat = fdopen(fd, "w");
      // if (fd >= 0) close(fd);
      if (dat == NULL) {
        fprintf(
            stderr,
            "ERROR: vecx cannot open %s.inf OR %s to save dat information\n",
            cart_filename, datfile);
        exit(1);
      }
      fprintf(stderr,
              "WARNING: vecx cannot open %s.inf to save dat information - "
              "saving to %s instead\n",
              cart_filename, datfile);
    }
    //  fprintf(stderr, "Quit #6\n");
    fprintf(dat, "V %X C %s\n", DatVersion, cart_filename);
    //  fprintf(stderr, "Quit #7\n");
    for (i = 0; i <= 0xFFFF; i++) {
      fprintf(dat, "%04X F %X ", i,
              GLOBAL_vecx->CPU
                  .F[i]);  // mark the ways in which ram/rom addresses are used.
      fprintf(
          dat, "D %X ",
          GLOBAL_vecx->CPU.DP[i]);  // Note the value of DP when executing code
                                    // at this address. 100 is unassigned 200 is
                                    // more than one value seen.
      fprintf(dat, "B %X ",
              GLOBAL_vecx->CPU
                  .IndexBase[i]);  // When loading table,x for any address, save
                                   // the value of 'table' for this address.
      fprintf(dat, "I %X",
              GLOBAL_vecx->CPU
                  .Index[i]);  // When loading table,x for any address, save the
                               // value of 'x' for this address.
      dup = 0;
      while ((i < 0xFFFF) &&
             (GLOBAL_vecx->CPU.F[i] == GLOBAL_vecx->CPU.F[i + 1]) &&
             (GLOBAL_vecx->CPU.DP[i] == GLOBAL_vecx->CPU.DP[i + 1]) &&
             (GLOBAL_vecx->CPU.IndexBase[i] ==
              GLOBAL_vecx->CPU.IndexBase[i + 1]) &&
             (GLOBAL_vecx->CPU.Index[i] == GLOBAL_vecx->CPU.Index[i + 1])) {
        dup += 1;
        i += 1;
      }
      fprintf(dat, " %d\n", dup);
    }
    //  fprintf(stderr, "Quit #8\n");
  }
#endif // NEVER

  //  fprintf(stderr, "Quit #9\n");
  exit(0);
}

static int debug_serial = 0;
int serial_port;

int main(int argc, char *argv[]) {
  FILE *binfile;
  Uint64 icycles, timeused, cyclesused;
  Uint64 now;
  int i;
  
  // I/O System:
  static int initialised = 0;
  char serial_device[32];
  int retVal;
  int rc;
  size_t numRead;
  char *s;

  progname = argv[0];
  if ((s = strrchr (progname, '/')) != NULL)
    progname = s + 1;		// Unix
  if ((s = strrchr (progname, '\\')) != NULL)
    progname = s + 1;		// M$
  if ((s = strrchr (progname, ']')) != NULL)
    progname = s + 1;		// Dec
  if ((s = strrchr (progname, ';')) != NULL)
    *s = '\0';			// Version no's
  if (((s = strrchr (progname, '.')) != NULL)
      && (strcasecmp (s, ".exe") == 0))
    *s = '\0';
  if (((s = strrchr (progname, '.')) != NULL)
      && (strcasecmp (s, ".com") == 0))
    *s = '\0';

  serial_port = 0;

#ifdef NEVER
  fprintf(stderr, "argc = %d, argv =", argc);
  for (i = 1; i < argc; i++) {
    fprintf(stderr, "[%d] \"%s\"", i, argv[i]);
  }
  fprintf(stderr, "\n");
#endif
  
  while (argc > 1) {
    if ((argc > 1) && (argv[1][1] >= '0') && (argv[1][1] <= '4') && (argv[1][2] == '\0')) { // -0 -1 -2 -3 -4
      serial_port = argv[1][1]-'0';
      argv++;
      argc--;
      continue;
    }
    if (strcmp (argv[1], "-d") == 0 ||
	strcmp (argv[1], "--debug") == 0) {
      argv++;
      argc--;
      debug = TRUE;
      continue;
    }
    if (strcmp(argv[1], "--help") == 0 ||
	strcmp(argv[1], "-h") == 0) {
      fprintf(stderr, "Usage: %s [options] [cart_file]\n", progname);
      fprintf(stderr, "Options:\n");
      fprintf(stderr, "  --help                    Display this help message\n");
      fprintf(stderr, "  --debug (or -d)           Debug on this console (not if -4 given also)\n");
      fprintf(stderr, "  -0                        Communicate over /dev/ttyUSB0 [default]\n");
      fprintf(stderr, "  -1                        Communicate over /dev/ttyUSB1\n");
      fprintf(stderr, "  -2                        Communicate over /dev/ttyAMA0\n");
      fprintf(stderr, "  -3                        Communicate over /dev/ttyS0\n");
      fprintf(stderr, "  -4                        Communicate over /dev/tty\n");
#ifdef Vectrex_SDL
      fprintf(stderr, "  --bios (or -b) <file>     Load bios file\n");
      fprintf(stderr, "  --overlay (or -o) <file>  Load overlay file\n");
      fprintf(stderr, "  --fullscreen (or -f)      Launch in fullscreen mode\n");
#endif 
      fprintf(stderr, "\n[cart_file] should be a 64K binary memory image and defaults to \"memory.bin\"\n");
      exit(0);
    }
#ifdef Vectrex_SDL
    if (strcmp(argv[1], "--bios") == 0 ||
	strcmp(argv[1], "-b") == 0) {
      bios_filename = argv[2];
      argv += 2;
      argc -= 2;
      continue;
    }
    if (strcmp(argv[1], "--overlay") == 0 ||
        strcmp(argv[1], "-o") == 0) {
      overlay_filename = argv[2];
      argv += 2;
      argc -= 2;
      continue;
    }
    if (strcmp(argv[1], "--fullscreen") == 0 ||
        strcmp(argv[1], "-f") == 0) {
      fullscreen = 1;
      argv++;
      argc--;
      continue;
    }
#endif
    if (argc >= 2 && (*argv[1] != '-')) {
      if (cart_filename) {
	fprintf(stderr, "max6809: only one memory file parameter allowed - can't have both %s and %s\n", cart_filename, argv[1]);
	exit(EXIT_FAILURE);
      }
      cart_filename = argv[1];
      argv++;
      argc--;
      continue;
    }
    if (argc > 1) {
      fprintf (stderr, "syntax: %s [-d] [-[01234]] "
#ifdef Vectrex_SDL
	       "[--bios <file>] [--overlay <file>] [--fullscreen] "
#endif
	       "[<cart_file>]\n", progname);
      exit (1);
    }
  }


  if (serial_port == 0) {
    strcpy(serial_device, "/dev/ttyUSB0");
    fprintf(stderr, "6809 Emulator started, using %s for I/O\n", serial_device);
    rc = system("stty -F /dev/ttyUSB0 speed 115200 pass8 cstopb -brkint -icrnl -imaxbel -opost -onlcr -isig -icanon -iexten -echo -echoe -echok -echoctl -echoke > /dev/null");
    if (rc) {
      fprintf(stderr, "%s: %s\n", progname, strerror(WEXITSTATUS(rc)));
      exit(EXIT_FAILURE);
    }
    system("stty < /dev/ttyUSB0");
  } else if (serial_port == 1) {
    strcpy(serial_device, "/dev/ttyUSB1");
    fprintf(stderr, "6809 Emulator started, using %s for I/O\n", serial_device);
    rc = system("stty -F /dev/ttyUSB1 speed 115200 pass8 cstopb -brkint -icrnl -imaxbel -opost -onlcr -isig -icanon -iexten -echo -echoe -echok -echoctl -echoke > /dev/null");
    if (rc) {
      fprintf(stderr, "%s: %s\n", progname, strerror(WEXITSTATUS(rc)));
      exit(EXIT_FAILURE);
    }
    system("stty < /dev/ttyUSB1");
  } else if (serial_port == 2) {
    strcpy(serial_device, "/dev/ttyAMA0");
    fprintf(stderr, "6809 Emulator started, using %s for I/O\n", serial_device);
    rc = system("stty -F /dev/ttyAMA0 speed 115200 pass8 cstopb -brkint -icrnl -imaxbel -opost -onlcr -isig -icanon -iexten -echo -echoe -echok -echoctl -echoke > /dev/null");
    if (rc) {
      fprintf(stderr, "%s: %s\n", progname, strerror(WEXITSTATUS(rc)));
      exit(EXIT_FAILURE);
    }
    system("stty < /dev/ttyAMA0");
  } else if (serial_port == 3) {
    strcpy(serial_device, "/dev/ttyS0");
    fprintf(stderr, "6809 Emulator started, using %s for I/O\n", serial_device);
    rc = system("stty -F /dev/ttyS0 speed 115200 pass8 cstopb -brkint -icrnl -imaxbel -opost -onlcr -isig -icanon -iexten -echo -echoe -echok -echoctl -echoke > /dev/null");
    if (rc) {
      fprintf(stderr, "%s: %s\n", progname, strerror(WEXITSTATUS(rc)));
      exit(EXIT_FAILURE);
    }
    system("stty < /dev/ttyS0");
  } else if (serial_port == 4) {
    strcpy(serial_device, "/dev/tty");
    //fprintf(stderr, "6809 Emulator started, using %s for I/O\n", serial_device);
    rc = system("stty -F /dev/tty speed 115200 pass8 cstopb -brkint -icrnl -imaxbel -opost -onlcr -isig -icanon -iexten -echo -echoe -echok -echoctl -echoke > /dev/null");
    if (rc) {
      fprintf(stderr, "%s: %s\n", progname, strerror(WEXITSTATUS(rc)));
      exit(EXIT_FAILURE);
    }
    //system("stty < /dev/tty");
    debug = 0;
  } else {
    fprintf(stderr, "Need a serial device: -0:/dev/ttyUSB0  -1:/dev/ttyUSB1  -2:/dev/ttyAMA0  -3:/dev/ttyS0  -4:/dev/tty\n");
    exit(EXIT_FAILURE);
  }
  if (serial_port != 4) {
    fprintf(stderr, "\nThis screen shows a copy of what is sent over the serial connection\n");
    fprintf(stderr, "It is *NOT* interactive and will not respond to any input except ^C\n\n");
  }
  
  Data_From_USB_Fd = open (serial_device, O_RDONLY);
  if (Data_From_USB_Fd >= 0) {
    Data_To_USB_Fd   = open (serial_device, O_WRONLY);
  }
  if ((Data_From_USB_Fd < 0) || (Data_To_USB_Fd < 0)) {
    fprintf(stderr, "Failure: open - %s\n", strerror(errno)); exit(EXIT_FAILURE);
  } else {
    int flags;
    if (debug_serial) fprintf(stderr, "Setting fd %d to O_NONBLOCK\n", Data_From_USB_Fd);
    flags = fcntl(Data_From_USB_Fd, F_GETFL, 0);
    fcntl(Data_From_USB_Fd, F_SETFL, flags | O_NONBLOCK);
  }

  //tcsendbreak (Data_To_USB_Fd, 0);

  
  // Emulator:
  
  // ACIA  at E000..E0FF
  // PIO   at E1XX..E1FF
  
  for (i = 0; i <= 0xFFFF; i++) {
    vectrex.memory[i] = 0;
    vectrex.disasm[i] = NULL;
  }
  vectrex.rom = &vectrex.memory[0x0000];
  vectrex.cart = &vectrex.memory[0x0000];
  vectrex.ram = &vectrex.memory[0x0000];

  vectrex.CPU.memory = vectrex.memory;
  vectrex.CPU.disasm = vectrex.disasm;

  //parse_args(argc, argv);

  if (!init()) quit();

  resize();
  //load_bios();
  //load_cart();
  //load_overlay();
  //e8910_init(&vectrex.PSG);
  vectrex.render = render;

  if (cart_filename == NULL) {
    cart_filename = "memory.bin";
  }
  binfile = fopen(cart_filename, "rb");
  if (binfile == NULL) {
    fprintf(stderr, "%s: %s - %s\n", argv[0], cart_filename, strerror(errno));
    exit(1);
  }
  vecx_reset(&vectrex);
  for (i = 0x0000; i < 0x10000; i++) vectrex.memory[i] = 0;
  for (i = 0x0000; i < 0x10000; i++) {
    int c = fgetc(binfile);
    if (c == EOF) break;
    vectrex.memory[i] = c;
  }
  fclose(binfile);
  //fprintf(stderr, "Memory populated from 0000 to %04x, reset vector = %02x%02x\n", i-1, vectrex.memory[0xFFFE], vectrex.memory[0xFFFF]);
  
  e6809_reset(&vectrex.CPU);
  init_console();
  // VECTREX_MHZ == 1500000 means 1500000 cycles take 1s to execute, or 1000000 uSec
  now = usecs();
  cyclesused = 0UL;
  struct timeval zero = { 0, 0 };
  for (;;) {

    FD_ZERO (&inFds);
    FD_SET (Data_From_USB_Fd, &inFds);

    retVal = select(Data_From_USB_Fd+1, &inFds, NULL, NULL, &zero);

    if (retVal < 0) {
      fprintf(stderr, "Failure: select - %s\n", strerror(errno));
      exit(EXIT_FAILURE);
    }
    
    if (FD_ISSET (Data_From_USB_Fd, &inFds)) { // data from user typing, arrives over USB
      if (debug_serial) fprintf(stderr, "Data available from USB\n");
      numRead = read (Data_From_USB_Fd, remote_buf, BUF_SIZE);
      if (numRead > 0) {
	int i;
	if (debug_serial) fprintf(stderr, "Got from USB:");
	for (i = 0; i < numRead; i++) {
	  if (debug_serial) fprintf(stderr, " %02x", remote_buf[i]);
	  q_push(remote_buf[i]);
	}
	if (debug_serial) fprintf(stderr, "\n");
      } else {
        fprintf(stderr, "No data read from USB?  numRead = %ld\n", (long)numRead);
      }
    }

    cyclesused += (Uint64)e6809_sstep(&vectrex.CPU, vectrex.VIA.ifr & 0x80, 0);
    if (cyclesused > VECTREX_MHZ/100UL) { // 1/100s resolution
      Uint64 savednow, virtualtime;
      virtualtime = cyclesused * 1000000UL / (Uint64)VECTREX_MHZ; // convert to uSec
      timeused = (savednow=usecs())-now;
      if (cyclesused > timeused) usleep(cyclesused-timeused); // sleep until we've used the same realtime that the real cpu would have needed
      //fprintf(stderr, "Cycles used: '%llu'  Time used: '%llu'\n", cyclesused, timeused);
      cyclesused = 0UL; now = usecs(); // savednow
    }

  }
  restore_console();
  //e8910_done(&vectrex.PSG);

  quit();

  return 0;
}
