// memimage64k -o hello.mem hello09.bin @0000 sbug.bin @F800

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <error.h>
#include <errno.h>

int hex(char *s, int *addr) {
  int i=0, c;
  while ((c = *s) != '\0') {
    if ('0' <= c && c <= '9')
      c -= '0';
    else if ('a' <= c && c <= 'f')
      c -= 'a'-10;
    else if ('A' <= c && c <= 'F')
      c -= 'A'-10;
    else
      return 0;
    i = (i<<4) | c;
    s += 1;
  }
  *addr = i;
  return 1;
}

static int quiet = 0;
static unsigned char mem[0x10000];
static int address;

int main(int argc, char **argv) {
  char *s, *progname = argv[0];
  char *option   = argv[1];
  char *outfilename = argv[2];
  FILE *outfile;
  char *infilename;
  char *loadaddr;
  FILE *infile;
  int rc;
  
  if ((s = strrchr (progname, '/')) != NULL)
    progname = s + 1;           // Unix
  if ((s = strrchr (progname, '\\')) != NULL)
    progname = s + 1;           // M$
  if ((s = strrchr (progname, ']')) != NULL)
    progname = s + 1;           // Dec
  if ((s = strrchr (progname, ';')) != NULL)
    *s = '\0';                  // Version no's
  if (((s = strrchr (progname, '.')) != NULL)
      && (strcasecmp (s, ".exe") == 0))
    *s = '\0';
  if (((s = strrchr (progname, '.')) != NULL)
      && (strcasecmp (s, ".com") == 0))
    *s = '\0';

  // memimage64k hello09.bin @0000 sbug.bin @F800
  if (argc == 1) {
    fprintf(stderr, "syntax: %s -o outfile.mem [file.bin @ADDR]*\n", progname);
    exit(EXIT_SUCCESS);
  }

  option = argv[1];
  outfilename = argv[2];

  if (strcmp(option, "-q") == 0) {
    quiet = 1;
    argc -= 1; argv += 1;
    option = argv[1];
    outfilename = argv[2];
  }

  if (strcmp(option, "-o") != 0) {
    fprintf(stderr, "syntax: %s -o outfile.mem [file.bin @ADDR]*\n", progname);
    exit(EXIT_SUCCESS);
  }
  
  outfile = fopen(outfilename, "wb");
  if (outfile == NULL) {
    fprintf(stderr, "%s: %s - %s\n", progname, outfilename, strerror(errno));
    exit(errno);
  }

  argc -= 2; argv += 2;

  for (;;) {
    if (argc == 1) break;
    if (argc >= 3) {
      infilename = argv[1];
      loadaddr = argv[2];
      address = 0x00000000;
      if ((loadaddr[0] != '@') || (strlen(loadaddr) != 5) || (!hex(loadaddr+1, &address))) {
        fprintf(stderr, "syntax: %s -o outfile.mem [file.bin @ADDR]*\n", progname);
        exit(EXIT_SUCCESS);
      }
      infile = fopen(infilename, "rb");
      if (infile == NULL) {
        fprintf(stderr, "%s: %s - %s\n", progname, infilename, strerror(errno));
        exit(errno);
      }
      rc = fread(&mem[address], 1, 0x10000, infile);
      fclose(infile);
      if ((address+rc) > 0x10000) {
        fprintf(stderr, "%s: Loading %04x bytes of %s at %04x writes past the end of 64K memory!\n", progname, (int)rc, infilename, (int)address);
        exit(EXIT_FAILURE);
      }
      if (!quiet) fprintf(stderr, "Loaded 0x%04x bytes %s to %04x\n", (int)rc, infilename, (int)address);
      argc -= 2; argv += 2;
    } else {
      fprintf(stderr, "syntax: %s -o outfile.mem [file.bin @ADDR]*\n", progname);
      exit(EXIT_SUCCESS);
    }
  }
  rc = fwrite(&mem[0], 1, 0x10000, outfile);
  fclose(outfile);
  if (!quiet) fprintf(stderr, "Wrote 0x%05x bytes to %s\n", rc, outfilename);
  exit(EXIT_SUCCESS);
  return 0;
}
