#ifndef __E6809_H
#define __E6809_H

#include <stdint.h>

#define DatVersion 0x1000          /* to ensure dat file compatibility - change whenever the format changes*/

typedef uint32_t FLAGS;            // 32 for now, may extend to 64 later if needed.

// Classify accesses
#define F_OPCODE             1    /* First byte of an opcode+operand */
#define F_OPERAND            2    /* Second and later bytes of opcode+operand */
#define F_DATA               4    /* fetched as data (eg 'LDA') */
#define F_DATA_LO            8    /* low byte of a double-byte fetch */
#define F_DATA_HI            16    /* high byte of a double-byte fetch */
#define F_CALL_TARGET        32
#define F_JUMP_TARGET        64
#define F_RETURN_TARGET      128
#define F_ILLEGAL            256    /* was fetched as an opcode or operand but did not decode correctly */
#define F_DATA_INDEXED       512    /* fetched as data using an indexed instruction.  May be part of a table. */
#define F_ADDRESS            1024    /* Not merely a 16-bit value but definitely one that points to code or data */
#define F_DATA_PACKET        2048
#define F_DATA_DIFFY_NUM     4096
#define F_DATA_DIFFY         8192
#define F_DATA_DUFFY_NUM_LEN 16384
#define F_DATA_DUFFY         32768
#define F_DATA_YX            65536
#define F_DATA_TUNE          131072
#define F_DATA_STRING80      262144
#define F_DATA_STRING        524288
#define F_LAST F_DATA_STRING

typedef struct M6809
{
	uint16_t reg_x, reg_y; /* index registers */
	uint16_t reg_u; /* user stack pointer */
	uint16_t reg_s; /* hardware stack pointer */
	uint16_t reg_pc; /* program counter */
	uint8_t reg_a, reg_b; /* accumulators */
	uint8_t reg_dp; /* direct page register */
	uint8_t reg_cc; /* condition codes */
	uint16_t irq_status; /* flag to see if interrupts should be handled (sync/cwai). */
	uint16_t *rptr_xyus[4];

    uint8_t *memory; //[0x10000]; // storage for complete address space incl ram, rom and cartridge
    FLAGS F[0x10000]; // mark the ways in which ram/rom addresses are used.
    uint16_t DP[0x10000]; // Note the value of DP when executing code at this address. 100 is unassigned 200 is more than one value seen.
    uint16_t IndexBase[0x10000]; // When loading table,x for any address, save the value of 'table' for this address.
    uint16_t Index[0x10000];     // When loading table,x for any address, save the value of 'x' for this address.
    char **disasm; // cached disassembly

    void *userdata;
    /* user defined read and write functions */
    uint8_t(*read8) (void *userdata, uint16_t address);
    void(*write8) (void *userdata, uint16_t address, uint8_t data);
} M6809;


void e6809_reset(M6809 *CPU);
uint16_t e6809_sstep(M6809 *CPU, uint16_t irq_i, uint16_t irq_f);

#endif
