// current gcc limitation:   variable%const for some values of const (probably when 2 bytes) always returns 0.

#define inline

#define NO_INDIRECTION

#ifdef NO_INDIRECTION
#define ACIACR (*(volatile unsigned char *)0xE004)
#define ACIASR (*(volatile unsigned char *)0xE004)
#define ACIATX (*(volatile unsigned char *)0xE005)
#define ACIARX (*(volatile unsigned char *)0xE005)
#else
volatile unsigned char *ACIACRp = (volatile unsigned char *)0xE004;
#define ACIACR (*ACIACRp)
volatile unsigned char *ACIASRp = (volatile unsigned char *)0xE004;
#define ACIASR (*ACIASRp)
volatile unsigned char *ACIATXp = (volatile unsigned char *)0xE005;
#define ACIATX (*ACIATXp)
volatile unsigned char *ACIARXp = (volatile unsigned char *)0xE005;
#define ACIARX (*ACIARXp)
#endif

#define CHAR_BIT 8
int _udivsi3(int n, int d) {
  const unsigned n_uword_bits = sizeof(int) * CHAR_BIT;
  int q;
  int r;
  unsigned int sr;
  if (d == 0) return 0;
  if (n == 0) return 0;
  sr = __builtin_clz(d) - __builtin_clz(n);
  if (sr > n_uword_bits - 1) return 0; // d > r
  if (sr == n_uword_bits - 1) return n; // d == 1
  ++sr;
  q = n << (n_uword_bits - sr);
  r = n >> sr;
  int carry = 0;
  for (; sr > 0; --sr) {
    r = (r << 1) | (q >> (n_uword_bits - 1));
    q = (q << 1) | carry;
    const int s = (int)(d - r - 1) >> (n_uword_bits - 1);
    carry = s & 1;
    r -= d & s;
  }
  q = (q << 1) | carry;
  return q;
}

int __umodsi3(int a, int b) {
  return a - _udivsi3(a, b) * b;
}

static inline void exit(int ignored) {
  asm(" jmp [0XF800]");
}

static inline int ttget(void) {
  //asm(" jmp [0XF806]");
  while ((ACIASR&1) == 0) /*wait*/;
  return ACIARX;
}

static inline int cr(void) {
  while ((ACIASR&2) == 0) /*wait*/;
  ACIATX = '\r';
  return '\r';
}

static inline int putchar(const int c) {
  while ((ACIASR&2) == 0) /*wait*/;
  ACIATX = c;
  if (c == '\n') cr();
  return c;
}

static inline int getchar(void) {
  int c = ttget();
  if (c == ('C'&31)) exit(0);
  if (c == '\r') c = '\n';
  putchar(c); // echo
  return c;
}

static inline void aciainit(void) { // Not needed if SBUG does hardware initialisation.
  ACIACR = 3;
  ACIACR = 0b00010101;
}

static inline int printstring(const char *s) {
  while (*s) (void)putchar(*s++);
  return 0;
}

static inline int puts(const char *s) {
  printstring(s); putchar('\n');
  return 0;
}


static void SHOW_NUM(char *dest, long num) { // left-aligned
#define show_digit(x) *dest++ = (char)((x)+'0')
  char digit, zeroes;

  // This replaces code that used divide by 10 and modulo 10.  *Ought* to be faster, though I haven't confirmed that yet!

  // handles full signed 16 bit range of -32768:32767  -  Uses negative numbers to avoid the issue of negating -32768

  if (num >= 0) num = -num; else *dest++ = '-';
  digit = 0;
  zeroes = 1; // CLRing is shorter
  // max 11 add/subtracts...
  if (num <= -20000) { num += 20000; digit += 2; zeroes = 0; }
  if (num <= -10000) { num += 10000; digit += 1; zeroes = 0; }
  if (!zeroes) show_digit(digit);
  digit = 0;
  if (num <= -8000) { num += 8000; digit += 8; zeroes = 0; } else if (num <= -4000) { num += 4000; digit += 4; zeroes = 0; }
  if (num <= -2000) { num += 2000; digit += 2; zeroes = 0; }
  if (num <= -1000) { num += 1000; digit += 1; zeroes = 0; }
  if (!zeroes) show_digit(digit);
  digit = 0;
  if (num <= -800) { num += 800; digit += 8; zeroes = 0; } else if (num <= -400) { num += 400; digit += 4; zeroes = 0; }
  if (num <= -200) { num += 200; digit += 2; zeroes = 0; }
  if (num <= -100) { num += 100; digit += 1; zeroes = 0; }
  if (!zeroes) show_digit(digit);
  digit = 0;
  if (num <= -80) { num += 80; digit += 8; zeroes = 0; } else if (num <= -40) { num += 40; digit += 4; zeroes = 0; }
  if (num <= -20) { num += 20; digit += 2; zeroes = 0; }
  if (num <= -10) { num += 10; digit += 1; zeroes = 0; }
  if (!zeroes) show_digit(digit);
  show_digit((char)-num);
  *dest = '\0';
}

static char num[16];
int write(int n, int p) { // places temporarily ignored.
  SHOW_NUM(num, n);
  printstring(num);
}

#define uint16_t unsigned long
#define WORDS 2547
// The word list has been sorted (roughly) by frequency of the words
// in common use.  (The sort program is in ~/src/IPA/spell/sort-by-freq.c)

// This way, the game can make sure that the words it suggests are
// not too obscure.
#define SIMPLE_WORDS 1023

// This array is a bit vector in which we mark whether a word is
// compatible with the scoring so far - if it isn't, that word is
// eliminated from the set of remaining possible solutions.
// This is the only data of any significant size that needs to
// be in RAM - all the other large arrays are stored in ROM.
static char eliminated[WORDS/8+1];

const char const *word[WORDS] = {
  "that", "this", "with", "from", "your", "have", "more", "will", 
  "home", "page", "free", "time", "they", "site", "what", "news", 
  "only", "when", "here", "also", "help", "view", "been", "were", 
  "some", "like", "than", "find", "date", "back", "list", "name", 
  "just", "over", "year", "into", "next", "used", "work", "last", 
  "most", "data", "make", "them", "post", "city", "such", "best", 
  "then", "good", "well", "info", "high", "each", "very", "book", 
  "read", "need", "many", "user", "said", "does", "mail", "full", 
  "life", "know", "days", "part", "real", "item", "must", "made", 
  "line", "send", "type", "take", "area", "want", "long", "code", 
  "show", "even", "much", "sign", "file", "link", "open", "case", 
  "same", "both", "game", "care", "down", "size", "shop", "text", 
  "rate", "form", "love", "john", "main", "call", "save", "card", 
  "jobs", "food", "sale", "teen", "room", "join", "west", "look", 
  "left", "team", "week", "note", "live", "plan", "cost", "test", 
  "come", "cart", "play", "less", "park", "side", "give", "sell", 
  "body", "east", "club", "road", "gift", "hard", "four", "blue", 
  "easy", "star", "hand", "keep", "baby", "term", "film", "head", 
  "cell", "self", "away", "once", "sure", "cars", "tell", "able", 
  "gold", "arts", "past", "five", "upon", "says", "land", "done", 
  "ever", "word", "bill", "talk", "nude", "kids", "true", "else", 
  "mark", "rock", "tips", "plus", "auto", "edit", "fast", "fact", 
  "unit", "meet", "feel", "bank", "risk", "town", "girl", "toys", 
  "golf", "loan", "wide", "sort", "half", "step", "none", "lake", 
  "fire", "chat", "loss", "face", "base", "near", "stay", "turn", 
  "mean", "king", "copy", "drug", "pics", "cash", "seen", "port", 
  "stop", "soon", "held", "mind", "lost", "tour", "menu", "hope", 
  "wish", "role", "came", "fine", "hour", "bush", "huge", "kind", 
  "move", "logo", "nice", "sent", "band", "lead", "went", "mode", 
  "fund", "male", "took", "song", "late", "fall", "idea", "tool", 
  "hill", "maps", "deal", "hold", "safe", "feed", "hall", "anti", 
  "ship", "paid", "hair", "tree", "thus", "wall", "wine", "vote", 
  "ways", "rule", "told", "feet", "sexy", "door", "cool", "uses", 
  "java", "pass", "fees", "skin", "ring", "boys", "deep", "rest", 
  "pool", "mini", "fish", "pack", "born", "race", "rape", "debt", 
  "core", "sets", "wood", "rent", "dark", "host", "fair", "gets", 
  "dead", "mike", "trip", "poor", "eyes", "farm", "lord", "hear", 
  "goes", "wife", "hits", "zone", "jack", "flat", "flow", "path", 
  "laws", "skip", "diet", "army", "gear", "lots", "firm", "jump", 
  "ball", "goal", "sold", "wind", "palm", "pain", "oral", "ford", 
  "edge", "root", "pink", "shot", "cold", "foot", "mass", "heat", 
  "wild", "miss", "task", "soft", "fuel", "walk", "wait", "rose", 
  "pick", "load", "tags", "guys", "cock", "drop", "rich", "seem", 
  "hire", "gave", "ones", "tits", "rank", "died", "inch", "snow", 
  "camp", "fill", "gone", "fort", "gene", "disc", "boat", "icon", 
  "ends", "cast", "felt", "soul", "aids", "flag", "atom", "iron", 
  "void", "disk", "desk", "vice", "duty", "bear", "gain", "lack", 
  "knew", "zoom", "blow", "clip", "wire", "tape", "acid", "cent", 
  "null", "zero", "roll", "bath", "font", "beta", "fail", "jazz", 
  "bags", "wear", "rare", "bars", "dual", "rise", "bird", "lady", 
  "dell", "seat", "bids", "toll", "cape", "mine", "whom", "math", 
  "dogs", "moon", "fear", "wars", "kept", "beat", "arms", "hide", 
  "slow", "nine", "spot", "grow", "rain", "onto", "bass", "hole", 
  "pets", "ride", "pair", "runs", "yeah", "evil", "euro", "peak", 
  "dick", "salt", "bell", "lane", "kill", "ages", "plug", "cook", 
  "bike", "lose", "seek", "kits", "soil", "matt", "exit", "keys", 
  "wave", "holy", "acts", "mesh", "dean", "poll", "bond", "jean", 
  "visa", "pure", "hell", "lens", "draw", "warm", "suck", "babe", 
  "crew", "legs", "rear", "node", "lock", "mile", "bowl", "tank", 
  "navy", "dish", "slot", "gray", "demo", "hate", "rice", "loop", 
  "vary", "milk", "boot", "push", "dear", "beer", "earn", "twin", 
  "bits", "suit", "chip", "char", "echo", "grid", "pull", "nick", 
  "plot", "pump", "exam", "beds", "grey", "bold", "scan", "aged", 
  "bulk", "cute", "para", "seed", "peer", "meat", "bang", "bone", 
  "bugs", "gate", "tone", "busy", "neck", "wing", "tiny", "rail", 
  "tube", "belt", "luck", "dial", "gang", "cake", "semi", "cafe", 
  "till", "shoe", "sand", "seal", "lies", "pipe", "deck", "thin", 
  "sick", "dose", "lets", "cats", "folk", "okay", "hist", "lift", 
  "mall", "fell", "yard", "pour", "dust", "butt", "kent", "adds", 
  "ward", "roof", "kiss", "rush", "yoga", "lamp", "glad", "wins", 
  "rack", "boss", "solo", "tall", "nova", "wake", "drum", "ease", 
  "orgy", "tabs", "pine", "tend", "gulf", "rick", "hunt", "mill", 
  "burn", "labs", "sole", "laid", "clay", "weak", "wise", "odds", 
  "sons", "leaf", "silk", "wolf", "fits", "kick", "meal", "hurt", 
  "slip", "cuts", "mars", "caps", "pill", "meta", "mint", "spin", 
  "wash", "aims", "soap", "axis", "guns", "hero", "punk", "duke", 
  "pace", "wage", "dawn", "coat", "doll", "reed", "mice", "temp", 
  "vast", "wrap", "mood", "quiz", "beam", "tops", "shut", "thou", 
  "mask", "coal", "lion", "beef", "hats", "surf", "hook", "cord", 
  "crop", "sing", "tons", "hang", "damn", "hood", "fame", "eggs", 
  "ruby", "stem", "drew", "tune", "corn", "puts", "grew", "trek", 
  "ties", "brad", "jury", "tail", "lawn", "soup", "byte", "nose", 
  "trim", "quit", "lung", "sees", "bull", "cole", "mart", "tale", 
  "docs", "coin", "fake", "cure", "arch", "bomb", "harm", "deer", 
  "oven", "noon", "mate", "chef", "isle", "slim", "spec", "midi", 
  "tied", "dale", "boob", "oils", "sept", "unto", "pays", "lang", 
  "stud", "fold", "slut", "pole", "bend", "glen", "lips", "pond", 
  "tire", "chad", "josh", "drag", "ripe", "rely", "nuts", "nail", 
  "span", "joke", "pads", "inns", "cups", "foam", "poem", "asks", 
  "bean", "bias", "swim", "loud", "rats", "stat", "thee", "ruth", 
  "pray", "pope", "jeep", "bare", "hung", "mono", "tile", "ciao", 
  "knee", "prep", "pros", "cant", "duck", "dive", "raid", "volt", 
  "dirt", "geek", "sink", "grip", "watt", "pins", "polo", "horn", 
  "frog", "logs", "snap", "swap", "flip", "buzz", "nuke", "boom", 
  "calm", "fork", "troy", "sims", "tray", "sage", "cave", "wool", 
  "eyed", "grab", "oops", "trap", "fool", "dies", "jail", "lace", 
  "ugly", "hart", "rows", "gods", "poly", "ears", "fist", "mere", 
  "cons", "taxi", "worn", "shaw", "expo", "deny", "trio", "cube", 
  "rugs", "crap", "fate", "oval", "tier", "earl", "cite", "mess", 
  "rope", "dump", "hose", "pubs", "mild", "clan", "sync", "mesa", 
  "hull", "shed", "memo", "tide", "funk", "reel", "bind", "rand", 
  "buck", "acre", "lows", "aqua", "pest", "reef", "jill", "sofa", 
  "tent", "hack", "dare", "hawk", "lamb", "junk", "poet", "epic", 
  "sake", "lean", "dude", "alto", "gore", "cult", "dash", "cage", 
  "ping", "flux", "rage", "rays", "acne", "undo", "halo", "gays", 
  "exec", "doom", "bite", "myth", "weed", "dice", "quad", "dock", 
  "mods", "hint", "buys", "pork", "barn", "fare", "bald", "mold", 
  "dame", "herb", "idle", "cove", "flex", "hash", "lazy", "carb", 
  "pens", "worm", "deaf", "mats", "mime", "keen", "peas", "owns", 
  "zinc", "guru", "levy", "bras", "pale", "gaps", "tear", "nest", 
  "gale", "idol", "moss", "cork", "dome", "heel", "yang", "dumb", 
  "feat", "glow", "oaks", "norm", "ware", "jade", "foul", "keno", 
  "seas", "pose", "goat", "sail", "bolt", "gage", "urge", "neon", 
  "ours", "lone", "cope", "lime", "kirk", "spas", "jets", "yarn", 
  "knit", "pike", "bent", "mama", "coil", "lily", "joey", "tilt", 
  "bead", "leak", "bulb", "iris", "stir", "pile", "wade", "pony", 
  "leap", "swan", "scat", "nets", "bash", "neat", "bump", "aunt", 
  "crow", "gram", "sits", "toes", "bust", "cola", "gems", "outs", 
  "foil", "amps", "tick", "vans", "glue", "cone", "olds", "ramp", 
  "pear", "clue", "duff", "scam", "beck", "tome", "prof", "sins", 
  "hath", "prop", "maid", "haha", "rods", "pigs", "maya", "deed", 
  "papa", "warn", "cops", "peso", "vent", "dots", "mugs", "vase", 
  "homo", "pier", "saga", "gran", "mayo", "lisp", "chin", "wont", 
  "hubs", "soda", "tomb", "prom", "cubs", "pulp", "pots", "slam", 
  "crib", "noun", "crab", "hike", "bore", "cows", "peel", "bake", 
  "heal", "tote", "sour", "sock", "bail", "tees", "mist", "verb", 
  "chic", "vest", "tens", "puma", "rant", "gill", "rude", "sang", 
  "curl", "deli", "vine", "vale", "limo", "rust", "avid", "bets", 
  "fade", "monk", "pooh", "torn", "heck", "bark", "lynx", "roms", 
  "odor", "flew", "apex", "curb", "deco", "coke", "webs", "prod", 
  "cane", "fury", "wore", "dues", "gigs", "plea", "prey", "lick", 
  "ness", "whip", "prog", "bins", "bout", "hank", "fond", "sore", 
  "fiat", "noel", "cans", "aide", "lieu", "reds", "knob", "dove", 
  "mack", "sung", "ally", "holt", "dull", "bees", "sigh", "mock", 
  "bats", "melt", "faux", "pant", "hype", "rite", "turf", "berg", 
  "muse", "tyne", "lust", "luna", "lien", "vein", "hale", "tiff", 
  "bait", "dent", "macs", "sums", "lame", "fuse", "pops", "amid", 
  "knot", "tate", "coop", "kris", "regs", "plum", "loft", "lump", 
  "halt", "aero", "bury", "duct", "lend", "pans", "ions", "rams", 
  "gown", "ruin", "tuna", "wipe", "haul", "duly", "obey", "mare", 
  "kite", "heap", "eats", "flea", "pals", "hail", "herd", "calf", 
  "rave", "jerk", "flap", "diva", "taps", "oath", "gala", "tang", 
  "riot", "fats", "ritz", "lush", "grim", "flop", "toss", "pity", 
  "woke", "cuff", "reps", "eddy", "cues", "colt", "offs", "jars", 
  "hips", "wigs", "hunk", "chap", "tele", "harp", "phat", "tack", 
  "sack", "peek", "secs", "sued", "toby", "opal", "toon", "viva", 
  "blew", "lava", "vail", "envy", "shah", "bots", "stag", "rash", 
  "boil", "vain", "maze", "rode", "amen", "warp", "zeta", "brew", 
  "kern", "coup", "fore", "alps", "subs", "limb", "limp", "mute", 
  "dart", "robe", "mast", "cozy", "comb", "alas", "chop", "onyx", 
  "hoop", "nope", "dire", "gull", "rims", "shin", "flaw", "pane", 
  "vibe", "teas", "anon", "ribs", "barb", "huns", "aura", "sect", 
  "nous", "mole", "owed", "buff", "clad", "fled", "aces", "lied", 
  "bloc", "hugs", "owls", "pimp", "gimp", "hemp", "inks", "wink", 
  "fife", "stub", "tort", "dams", "mach", "spur", "skis", "pact", 
  "lulu", "vita", "ibis", "mule", "kiwi", "duel", "mead", "gong", 
  "bays", "gaze", "hulk", "cyan", "axes", "haze", "pits", "ants", 
  "tuck", "slab", "lure", "kart", "ling", "tubs", "mags", "hers", 
  "grin", "spit", "acme", "rove", "chow", "aria", "dirk", "scar", 
  "dorm", "pies", "saws", "dine", "slap", "jaws", "ante", "bred", 
  "tout", "hare", "tory", "veil", "peck", "helm", "jess", "jays", 
  "chew", "teak", "sane", "dune", "mana", "lest", "blur", "punt", 
  "mojo", "dime", "carp", "dusk", "lent", "aloe", "puff", "claw", 
  "tyre", "wand", "taco", "weld", "writ", "typo", "damp", "fern", 
  "lore", "gyms", "arid", "jive", "spun", "loch", "balm", "figs", 
  "chit", "lays", "drip", "mans", "pint", "maxi", "liar", "opus", 
  "mage", "teal", "bard", "reap", "tidy", "vets", "bows", "beau", 
  "rang", "laps", "pong", "vows", "soar", "flee", "woes", "stew", 
  "hush", "lama", "akin", "dads", "hays", "soak", "hymn", "quay", 
  "weir", "dram", "burr", "slug", "lira", "jams", "dyes", "yell", 
  "fins", "rake", "suns", "bunk", "stab", "dyke", "seam", "omit", 
  "darn", "ammo", "linn", "veto", "clam", "nerd", "guts", "bans", 
  "buds", "sire", "ding", "coma", "thug", "sham", "loaf", "skid", 
  "poop", "whey", "curt", "moth", "mash", "germ", "atop", "kemp", 
  "puck", "moor", "lube", "sway", "toad", "eros", "malt", "pods", 
  "clap", "paws", "poke", "dope", "kiln", "jock", "dyed", "wasp", 
  "fuss", "swat", "sine", "bray", "roar", "gall", "jugs", "apes", 
  "slit", "cine", "beep", "toms", "slid", "nana", "raft", "brow", 
  "vars", "fink", "deem", "flax", "tart", "tram", "iced", "bile", 
  "heir", "loco", "feds", "loom", "grub", "slew", "duet", "ohms", 
  "muck", "hoax", "wary", "riff", "mace", "lids", "pore", "oily", 
  "whoa", "joys", "grit", "nays", "sinh", "sues", "tame", "oats", 
  "wilt", "ting", "emit", "rink", "numb", "scot", "pave", "airy", 
  "hobo", "hops", "airs", "dole", "wick", "owes", "peat", "tofu", 
  "twat", "sunk", "perm", "lair", "lacy", "wren", "arcs", "ayes", 
  "heed", "boon", "pawn", "tore", "dunk", "snip", "mutt", "ouch", 
  "dips", "digs", "womb", "joss", "kohl", "pups", "lido", "daft", 
  "gilt", "trad", "taxa", "sled", "elan", "refs", "coax", "lees", 
  "alum", "doth", "plow", "smut", "foes", "blot", "imam", "hove", 
  "rial", "sewn", "rift", "peep", "judo", "mays", "oboe", "lobe", 
  "gosh", "mica", "lads", "sumo", "urea", "zeal", "bong", "gent", 
  "hind", "ache", "yoke", "tins", "loot", "gasp", "tint", "silt", 
  "lint", "blob", "roam", "stun", "trey", "fang", "weep", "giga", 
  "ergo", "moan", "sank", "bale", "smog", "keel", "fart", "cosy", 
  "seer", "vial", "huff", "gust", "musk", "lice", "skim", "rune", 
  "welt", "veal", "rags", "posh", "mink", "itch", "dill", "rigs", 
  "moot", "agar", "rein", "buoy", "hive", "newt", "dung", "lark", 
  "hoes", "ruff", "zoos", "scum", "bran", "faro", "giro", "cots", 
  "fret", "gums", "glib", "gees", "vile", "brio", "peri", "loci", 
  "hula", "foxy", "putt", "yeas", "cram", "hens", "hogs", "fray", 
  "gags", "rips", "sash", "lire", "raja", "boar", "mite", "spar", 
  "redo", "nome", "brat", "wits", "anew", "loon", "hone", "sips", 
  "wort", "nuns", "tuba", "jest", "mort", "repo", "gnus", "jolt", 
  "skew", "cyst", "fowl", "pate", "snug", "orca", "nook", "rosy", 
  "howl", "hobs", "ogre", "sped", "yogi", "pats", "dojo", "gait", 
  "meek", "stow", "bibs", "cabs", "hump", "beet", "geld", "pegs", 
  "meow", "lash", "ills", "omen", "brig", "tuff", "coda", "defy", 
  "foss", "kilt", "whit", "coos", "vino", "clog", "yeti", "sill", 
  "buns", "nils", "lain", "holm", "tots", "tors", "furl", "hogg", 
  "muon", "afar", "fawn", "etch", "flue", "emir", "mobs", "gout", 
  "rubs", "bobs", "bane", "thaw", "eels", "shun", "daze", "trot", 
  "woof", "davy", "wham", "begs", "mull", "bode", "urns", "quid", 
  "kilo", "nods", "parr", "dork", "sari", "mitt", "yawn", "feud", 
  "brut", "gist", "zips", "tact", "rife", "huts", "silo", "eras", 
  "mend", "mums", "tern", "lass", "rota", "jute", "prim", "pang", 
  "swag", "opts", "mano", "furs", "whim", "pied", "nigh", "ales", 
  "sump", "zest", "wisp", "sift", "ooze", "fume", "brag", "muff", 
  "snag", "yank", "shag", "hues", "lags", "spat", "ices", "ahem", 
  "wavy", "crux", "obit", "slag", "skit", "kelp", "vamp", "iota", 
  "bled", "pita", "slum", "fuzz", "toil", "garb", "pars", "twig", 
  "wept", "edgy", "jigs", "burg", "rook", "pall", "trig", "moat", 
  "swam", "hazy", "hoof", "frag", "vise", "lute", "pram", "brim", 
  "chum", "glut", "neve", "sacs", "soot", "roux", "vane", "mane", 
  "kink", "rhea", "sown", "beak", "rung", "twas", "loam", "romp", 
  "czar", "mush", "foal", "gogo", "noes", "pail", "loin", "leer", 
  "nave", "hows", "leek", "puss", "oars", "clot", "soya", "joes", 
  "morn", "glug", "rout", "ploy", "magi", "puke", "glee", "hams", 
  "hoot", "pina", "bott", "kale", "huck", "juke", "jinx", "tics", 
  "tora", "fido", "aver", "pugs", "tala", "cols", "milt", "wart", 
  "naps", "gush", "ogle", "buss", "hiss", "rusk", "gory", "bony", 
  "moll", "mews", "cask", "brie", "oxen", "bums", "mops", "nota", 
  "fend", "veer", "nips", "debs", "glob", "ream", "ores", "boll", 
  "shim", "ides", "gnat", "amok", "arty", "smug", "dons", "yule", 
  "lugs", "bate", "pert", "tsar", "cosh", "burp", "wert", "cusp", 
  "bade", "revs", "yolk", "boos", "cull", "maul", "ahoy", "haar", 
  "poms", "yelp", "gyro", "luge", "sear", "egos", "pare", "viol", 
  "tarp", "puny", "miso", "fizz", "foci", "drab", "flak", "yuck", 
  "quod", "ably", "lard", "hypo", "swab", "rind", "rote", "ruse", 
  "muss", "spud", "croc", "elms", "feta", "onus", "bier", "lull", 
  "lino", "shew", "elks", "tutu", "raps", "alms", "biff", "swot", 
  "asps", "mote", "wold", "boas", "idem", "hark", "thud", "weds", 
  "perk", "firs", "rump", "pent", "sago", "rive", "tabu", "gawk", 
  "hock", "heft", "lewd", "dupe", "bogs", "tosh", "loup", "swig", 
  "fete", "duns", "fain", "pips", "sirs", "cowl", "slat", "goon", 
  "pica", "taut", "kola", "snob", "crud", "tine", "racy", "awry", 
  "lode", "spay", "toke", "clef", "pelt", "wadi", "goof", "ankh", 
  "dank", "zany", "toga", "puns", "hilt", "roan", "nags", "wily", 
  "dodo", "cogs", "talc", "pomp", "noms", "spew", "gulp", "lurk", 
  "pend", "toot", "snot", "hurl", "dory", "nary", "ibex", "orbs", 
  "razz", "zing", "toed", "scab", "phew", "wail", "crag", "tass", 
  "eves", "minx", "dens", "quin", "purr", "nits", "oust", "bask", 
  "gash", "dubs", "odes", "wane", "oyez", "fads", "ovum", "dene", 
  "slaw", "nabs", "whig", "tugs", "meld", "lyre", "deft", "husk", 
  "hick", "honk", "turd", "cuss", "pout", "mire", "nett", "saps", 
  "shiv", "balk", "sark", "robs", "idly", "ewes", "gast", "waxy", 
  "loos", "mien", "rede", "abet", "kerb", "curd", "barf", "blip", 
  "icky", "doss", "sows", "wimp", "moos", "oryx", "yams", "rend", 
  "kine", "rots", "rood", "dado", "whet", "doer", "seep", "sods", 
  "sobs", "purl", "sawn", "muds", "wean", "juju", "dink", "imps", 
  "pupa", "suds", "okra", "bren", "snub", "prat", "dims", "derm", 
  "scud", "sops", "slur", "uric", "tusk", "twee", "shoo", "sate", 
  "yore", "slop", "bozo", "twos", "hake", "gaol", "tarn", "fags", 
  "tare", "gaga", "flay", "fess", "hots", "sett", "ilex", "whee", 
  "awed", "twit", "ruck", "slog", "flog", "topi", "wile", "duds", 
  "deet", "rime", "goad", "spry", "coot", "jabs", "oink", "pews", 
  "brae", "jots", "ursa", "plop", "grog", "mell", "runt", "cede", 
  "teem", "bonk", "duos", "suet", "inky", "hewn", "vole", "syne", 
  "flan", "axed", "jags", "quip", "shod", "iffy", "saki", "wove", 
  "bide", "bevy", "flit", "dour", "kegs", "nosy", "dabs", "posy", 
  "ghee", "vend", "abut", "eons", "luff", "teat", "dais", "pith", 
  "gaff", "chug", "laud", "buts", "goer", "peed", "bods", "peal", 
  "coir", "kook", "nobs", "vats", "trod", "nape", "hols", "weft", 
  "teds", "lase", "goos", "gins", "reek", "wets", "kens", "hums", 
  "vacs", "bung", "rapt", "otic", "leet", "pees", "rasp", "doze", 
  "wonk", "tats", "dint", "ells", "gape", "sone", "wows", "bole", 
  "craw", "tuft", "toft", "flam", "nibs", "ails", "cued", "slob", 
  "glum", "hems", "serf", "boxy", "woks", "hued", "hoar", "jamb", 
  "whys", "fens", "cads", "abed", "ruts", "ulan", "prow", "suss", 
  "dace", "wags", "maim", "kudu", "tush", "espy", "rums", "tipi", 
  "curs", "zits", "gunk", "vies", "lope", "errs", "roup", "faze", 
  "ajar", "fobs", "irks", "nosh", "byes", "lout", "mete", "wads", 
  "swop", "togs", "jibe", "haws", "gobs", "raze", "hasp", "wain", 
  "pyre", "apse", "souk", "tope", "lorn", "nock", "tows", "dewy", 
  "weal", "koan", "puce", "epee", "stet", "tars", "amyl", "teed", 
  "yobs", "dibs", "emus", "yurt", "caws", "lade", "wend", "wiry", 
  "snog", "tuts", "exes", "dray", "haft", "iglu", "fogs", "daws", 
  "loth", "trow", "sibs", "blab", "gyre", "frit", "achy", "arum", 
  "sags", "lath", "woos", "gild", "wino", "wyle", "lank", "skat", 
  "aced", "plod", "faun", "hebe", "tyro", "rill", "tamp", "oast", 
  "gird", "gnaw", "pock", "tyke", "scag", "ebbs", "laze", "drat", 
  "clod", "waif", "kith", "sulk", "cobs", "dhow", "kips", "dolt", 
  "rids", "knap", "adze", "clew", "ewer", "floe", "oldy", "guff", 
  "nubs", "lilt", "wees", "ashy", "efts", "lobs", "jogs", "zaps", 
  "dews", "daub", "ulna", "adit", "murk", "wist", "coif", "flab", 
  "ouzo", "ossa", "toff", "vair", "gads", "khat", "yaks", "skua", 
  "veep", "clop", "wive", "zags", "kins", "tike", "ghat", "bilk", 
  "vied", "sorb", "fibs", "bops", "snit", "nowt", "erne", "rues", 
  "dote", "ergs", "scow", "mown", "pash", "sews", "rile", "nide", 
  "pawl", "neap", "kapa", "arks", "rhos", "moke", "frap", "waft", 
  "jeer", "impi", "skee", "swum", "yawl", "agog", "doff", "wynd", 
  "hags", "egad", "juts", "gabs", "etui", "mope", "ogee", "stob", 
  "maws", "bawl", "woad", "busk", "sloe", "lops", "roue", "yaws", 
  "tuns", "ecus", "loll", "chaw", "tufa", "jowl", "zonk", "kadi", 
  "utas", "eggy", "boff", "gelt", "tzar", "conk", "skag", "unco", 
  "wych", "avow", "hews", "dozy", "poky", "roil", "mows", "oohs", 
  "stye", "zebu", "miff", "boyo", "grot", "byre", "raki", "eked", 
  "gamp", "harl", "whin", "idyl", "vagi", "corm", "wens", "ulva", 
  "neuk", "ulus", "dins", "bawd", "fops", "hoed", "lour", "gibe", 
  "hadj", "mazy", "awls", "moil", "phut", "uvea", "hies", "sots", 
  "upas", "hied", "jape", "skeg", "dunt", "dugs", "braw", "hods", 
  "yips", "rued", "sups", "nark", "puds", "yaps", "oafs", "odic", 
  "taws", "fugs", "iamb", "trug", "goys", "orra", "frug", "peke", 
  "prig", "gley", "jilt", "auks", "ilea", "albs", "ekes", "keek", 
  "cloy", "gybe", "oxid", "eyas", "joky", "spiv", "gawp", "aped", 
  "lyes", "aery", "awes", "tivy", "scut", "yowl", "surd", "oozy", 
  "zigs", "pleb", "frow", "enow", "zees", "drib", "winy", "jink", 
  "drub", "tups", "shmo", "mump", "smew", "wost", "sned", "sild", 
  "nogs", "sizy", "girn", "dopy", "snib", "pipy", "zeds", "xyst", 
  "ogam", "knar", "knur"
};

// apologies for the Bowdlerization of maybe half a dozen words - the word
// list came from scratch.mit.edu which has strict rules about inappropriate
// language for young children.

uint16_t recorded_word[12]; // record the number of the word for each guess.
int recorded_score[12]; // if we can spare the space, use this for verifying scoring.

const char const *prompt[12] = {
  "Your first guess: ",
  "Your second guess: ",
  "Your third guess: ",
  "Your fourth guess: ",
  "Your fifth guess: ",
  "Your sixth guess: ",
  "Your seventh guess: ",
  "Your eighth guess: ",
  "Your ninth guess: ",
  "Your tenth guess: ",
  "Your penultimate guess: ",
  "Your final guess: ",
};

#ifndef TRUE
#define TRUE (0==0)
#define FALSE (0!=0)
#endif

unsigned int _x;
unsigned int _a;
unsigned int _b; 
unsigned int _c; 

void initRandom(unsigned int s1,unsigned int s2,unsigned int s3, unsigned int x0)
{
	_x = x0;
	_a = s1;
	_b = s2;
	_c = s3;
	_x++;
	_a = (_a^_c^_x);
	_b = (_b+_a);
	_c = ((_c+(_b>>1))^_a);
}

uint16_t random8()
{
	_x++;
	_a = (_a^_c^_x);
	_b = (_b+_a);
	_c = ((_c+(_b>>1))^_a);
	return (uint16_t)_c;
}

uint16_t random(void) {
  int result;
  result = ((random8() << 8) | random8()) ^ (random8()<<4);
  return result;
}

static inline int isalpha(int c) {
  return ((('a' <= c) && (c <= 'z')) || (('A' <= c) && (c <= 'Z')));
}
static inline int isupper(int c) {
  return (('A' <= c) && (c <= 'Z'));
}
static inline int islower(int c) {
  return (('a' <= c) && (c <= 'z'));
}
static inline int tolower(int c) {
  return c-'A'+'a';
}


static int score(const char *hidden_word, const char *guess, int *bulls, int *cows) {
  char my_word[5], my_guess[5];
  int p;
  for (p = 0; p < 4; p++) {      // make writable
    my_word[p] = hidden_word[p];
    my_guess[p] = guess[p];
  }
  
  *bulls = 0; *cows = 0;
  for (p = 0; p < 4; p++) {
    if (my_guess[p] == my_word[p]) {
      // remove from further consideration:
      my_word[p] = ' ';
      my_guess[p] = '_';
      *bulls += 1;
    }
  }
  if ((my_guess[0] != '_') && (my_guess[0] == my_word[p=1] || my_guess[0] == my_word[p=2] || my_guess[0] == my_word[p=3])) {
    my_word[p] = ' '; *cows += 1;
  }
  if ((my_guess[1] != '_') && (my_guess[1] == my_word[p=0] || my_guess[1] == my_word[p=2] || my_guess[1] == my_word[p=3])) {
    my_word[p] = ' '; *cows += 1;
  }
  if ((my_guess[2] != '_') && (my_guess[2] == my_word[p=0] || my_guess[2] == my_word[p=1] || my_guess[2] == my_word[p=3])) {
    my_word[p] = ' '; *cows += 1;
  }
  if ((my_guess[3] != '_') && (my_guess[3] == my_word[p=0] || my_guess[3] == my_word[p=1] || my_guess[3] == my_word[p=2])) {
    my_word[p] = ' '; *cows += 1;
  }
  return *bulls * 5 + *cows;
}

static void eliminate(uint16_t word_index) { // use a bit array to keep track of eliminated words.
  uint16_t index;
  int bit;
  index = word_index >> 3;
  bit = 1 << (word_index&7); // assuming 1<<0 == 1 is implemented correctly.
  eliminated[index] |= bit;
}

static int is_eliminated(uint16_t word_index) {
  uint16_t index;
  int bit;
  index = word_index >> 3;
  bit = 1 << (word_index&7);
  return (eliminated[index] & bit) != 0;
}

void dump_marks(int guesses) {
  int i;
  printstring("You can check the scoring here.\n\n    Word Score\n");
  for (i = 0; i < guesses; i++) {
    int bulls, cows, score;
    uint16_t w;
    w = recorded_word[i];
    score = recorded_score[i];
    cows = score%5; bulls = score/5;
    //printf("%2d: %s ", i+1, word[w]);
    write(i+1,0); printstring(": "); printstring(word[w]); putchar(' ');    
    while (bulls-- > 0) putchar('+');
    while (cows-- > 0) putchar('?');
    putchar('\n');
  }
  putchar('\n');
}

void check_scores(const char *hidden_word, int guesses) {
  int i;
  printstring("         Score   Actual\n");
  printstring("    Word By You  Score\n");
  for (i = 0; i < guesses; i++) {
    int chars, bulls, cows, my_score, your_score;
    uint16_t w;
    w = recorded_word[i];
    your_score = recorded_score[i];
    cows = your_score%5; bulls = your_score/5; chars = 4-(bulls+cows);
    //printf("%2d: %s ", i+1, word[w]);
    write(i+1,2); printstring(": "); printstring(word[w]); putchar(' ');    
    while (bulls-- > 0) putchar('+');
    while (cows-- > 0) putchar('?');
    while (chars-- > 0) putchar(' ');
    printstring("    ");
    my_score = score(hidden_word, word[w], &bulls, &cows); chars = 4-(bulls+cows);
    while (bulls-- > 0) putchar('+');
    while (cows-- > 0) putchar('?');
    while (chars-- > 0) putchar(' ');
    if (your_score != my_score) printstring("   <--- looks like you mis-scored that one...");
    putchar('\n');
  }
  putchar('\n');
}

#define EOF (-1)
#define COMPUTER 0
#define HUMAN 1
int main(void) {
  uint16_t w, temp, p, words = WORDS; // sizeof(word)/sizeof(word[0]);
  int bulls, cows, guesses, humans_score, turn;
  char *saved_word;
  char guess[5]; // need to be writable

  //aciainit();
  putchar('\r');
  
  (void)puts("Welcome to Word Mastermind\n\nPress SPACE to start.\n\n"); // Might ask for the player's name to make it less artifical.
  for (;;) {
    // scramble the random seed by using typing delay.
    _x += 3;
    _a += 5;
    _b += 7;
    _c += 11;
    if ((ACIASR&1) != 0) {
      if (ACIARX == ' ') break;
    }
  }
  initRandom(_x,_a,_b,_c);
  
  turn = HUMAN;
  printstring("\nI'll pick a word first, it'll give you a chance to learn the scoring system\n"
         "before you have to score my guesses...\n\nIMPORTANT NOTE TO REMEMBER: All words have 4 letters.\n\n");

 restart:
  turn = (HUMAN+COMPUTER) - turn;
  if (turn == HUMAN) {
    int valid = 0, awarded_score;
    char *computers_guess;
    for (w = 0; w < 2564/8+1; w++) eliminated[w] = 0;
    printstring("I want you to think of a word with FOUR LETTERS and I'll try to\n"
           "guess it.  For every letter in your word that matches a letter in the\n"
           "same place in my word, you award me a '+'.  For each of the remaining\n"
           "letters in your word, if they appear anywhere in my word, You\n"
           "give me a '?'.\n\n");
    temp = random()&SIMPLE_WORDS;
    computers_guess = (char *)word[temp];
    guesses = 0;
    // printf("My first guess is: %s\n\n(use '+' and '?', or '0' if none.)\n");
    printstring("My first guess is: "); printstring(computers_guess); printstring("\n\n(use '+' and '?', or '0' if none.)\n"); 
    recorded_word[guesses] = temp;
    guesses += 1;
  next_try:
    printstring("What's my score?: ");
    valid = FALSE; bulls = 0; cows = 0; 
    for (;;) {
      int c = '\n';
      c = getchar();
      if (c == EOF || c == ('D'&31)) {
        printstring("You give up?  OK, I win. Your turn to guess next.\n");
        dump_marks(guesses);
        goto restart;
      }
      if (c == '+' || c == '?' || c == '0') valid = valid | TRUE;
      if (c == '+') bulls += 1;
      if (c == '?') cows += 1;
      if (c == '\n') {
        if (!valid) {
          printstring("Remember, for every letter in your word that matches a letter in the\n"
                 "same place in my word, you award me a '+'.  For each of the remaining\n"
                 "letters in your word, if they appear anywhere in my word, You\n"
                 "give me a '?'.  Enter '0' if I didn't guess any letters correct at all.\n\n"
                 "What's my score?: ");
          continue;
        } else break;
      }
    }
    awarded_score = bulls*5 + cows;
    recorded_score[guesses-1] = awarded_score;
    if (bulls == 4) {
      // printf("Woo hoo!  I won, in %d guesses!\n\n", guesses);
      printstring("Woo hoo!  I won, in "); write(guesses,0); printstring(" guesses!\n\n"); 
      dump_marks(guesses);//
      goto restart;
    }
    if (guesses == 12) {
      printstring("Well, that was 12 guesses and I didn't get it.\n");
     retype_humans_secret_word:
      for (;;) {
        printstring("What was your word?\n");
        p = 0;
        for (;;) {
          int c = '\n';
          c = getchar();
          if (c == EOF || c == ('D'&31)) {
            printstring("\nYou quit? OK.\n");
            dump_marks(guesses);//
            goto restart;
          }

          if (isalpha(c)) {
            if (isupper(c)) c = tolower(c);

            if (p < 4) {
              guess[p++] = (char)c;
            } else {
              p = 5; // signal to '\n' code that the word was too long
            }
          } else if (c == '\n') {
            if (p != 4) {
              printstring("Your word had to be a word of four letters.  Try again.\n");
              goto retype_humans_secret_word;
            }
            break;
          } else {
            // silently ignore bad character
          }
        }
        // was it a valid word?
        for (w = 0; w < words; w++) {
          for (p = 0; p < 4; p++) {
            if (guess[p] != word[w][p]) break;
            if (p==3) goto was_valid;
          }
        }
        // printf("There's the problem... \"%s\" was not a word you could use.  You lose.\n", guess);
        printstring("There's the problem... \""); printstring(guess); printstring("\" was not a word you could use.  You lose.\n"); 
      was_valid:
        printstring("I think there may have been a scoring error.  Let's check:\n\n");
        check_scores(guess, guesses);
        goto restart;
      }
      dump_marks(guesses);//
      goto restart;
    }
    // printf("OK, so I scored %d bulls and %d cows.\n", bulls, cows);
    printstring("OK, so I scored ");
    if (bulls == 0) printstring("no"); else write(bulls,0);
    printstring(" bull");
    if (bulls != 1) putchar('s');
    printstring(" and ");
    if (cows == 0) printstring("no"); else write(cows,0);
    printstring(" cow");
    if (cows != 1) putchar('s');
    printstring(".\n"); 
    for (w = 0; w < words; w++) {
      int xbulls, xcows; // check
      // assume the hidden word was word[i].
      // If it were, is the score the player awarded for my guess compatible with that word?
      int test_score = score(word[w], computers_guess, &xbulls, &xcows);
      if (test_score != awarded_score) eliminate(w);
      // when you are down to 3 bulls, and there are multiple choices left for the last letter,
      // human players usually do something smarter than try them all one by one.
    }
    temp = 0;
    for (w = 0; w < words; w++) {
      if (!is_eliminated(w)) {
        temp += 1;
      }
    }
    putchar('\n');
    if (temp == 0) {
      printstring("OK, I'm out of ideas.\n"
             "I'm pretty sure that either you're using a word I don't know,\n"
             "or you mis-calculated one of the scores you awarded me earlier.\n");
    re_enter_humans_secret_word:
      for (;;) {
        printstring("What was your word?\n");
        p = 0;
        for (;;) {
          int c = '\n';
          c = getchar();
          if (c == EOF || c == ('D'&31)) {
            printstring("\nYou quit?  OK.\n");
            dump_marks(guesses);//
            goto restart;
          }

          if (isalpha(c)) {
            if (isupper(c)) c = tolower(c);

            if (p < 4) {
              guess[p++] = (char)c;
            } else {
              p = 5; // signal to '\n' code that the word was too long
            }
          } else if (c == '\n') {
            if (p != 4) {
              printstring("Your word has to be a word of four letters.  Try again.\n");
              goto re_enter_humans_secret_word;
            }
            break;
          } else {
            // silently ignore bad character
            // printf("\nbad character '%c'\n", c);
            printstring("\nbad character '"); putchar(c); printstring("'\n"); 
          }
        }
        // was it a valid word?
        for (w = 0; w < words; w++) {
          for (p = 0; p < 4; p++) {
            if (guess[p] != word[w][p]) break;
            if (p==3) goto was_indeed_valid;
          }
        }
        // printf("There's the problem... \"%s\" was not a word you could use.  You lose.\n", guess);
        printstring("There's the problem... \""); printstring(guess); printstring("\" was not a word you could use.  You lose.\n"); 
        goto restart;
      was_indeed_valid:
        printstring("I think there may have been a scoring error.  Let's check:\n\n");
        check_scores(guess, guesses);
        goto restart;
      }
    } else if (temp == 1) {
      printstring("I think I've guessed it!\n\n");
    // } else {
      // // printf("There are %d possible words this could be!\n\n", (int)temp);
      // printstring("There are "); write((int)temp,0); printstring(" possible words this could be!\n\n"); 
    }
    // find non-eliminated word 'temp'...
    temp = random()%temp; // eg random(4) is 0..3
    computers_guess = "zzzz"; // delete when sure OK
    for (w = 0; w < words; w++) {
      if (!is_eliminated(w)) {
        if (temp == 0) {
          computers_guess = (char *)word[temp = w];
          break;
        }
        temp -= 1;
      }
    }

    bulls = 0; cows = 0;
    // printf("My next guess is: %s\n\n(use '+' and '?', or '0' if none.)\n", computers_guess);
    printstring("My next guess is: "); printstring(computers_guess); printstring("\n\n(use '+' and '?', or '0' if none.)\n"); 
    recorded_word[guesses] = temp;
    guesses += 1;
    goto next_try;
  }

  guesses = 0;
  printstring("I'm thinking of a word.  You can offer valid words as guesses.\n"
         "For every letter in your word that matches a letter in the same\n"
         "place in my word, I'll award you a '+'.  For each of the remaining\n"
         "letters in your word, if they appear anywhere in my word I'll\n"
         "give you a '?'.\n");
  temp = random()&SIMPLE_WORDS;
  saved_word = (char *)word[temp];
 retype_humans_guess:
  guess[4] = '\0';
  for (;;) {
    p=0; bulls=0; cows=0;
    // printf("%s", prompt[guesses]);
    printstring(prompt[guesses]); 
    for (;;) {
      int c = '\n';
      c = getchar();
      if (c == EOF || c == ('D'&31)) {
        // printf("\nYou quit?  The word was \"%s\"\n", saved_word);
        printstring("\nYou quit?  The word was \""); printstring(saved_word); printstring("\"\n"); 
        dump_marks(guesses);//
        goto restart;
      }

      if (isalpha(c)) {
        if (isupper(c)) c = tolower(c);

        if (p < 4) {
          guess[p++] = (char)c;
        } else {
          p = 5; // signal to '\n' code that the word was too long
        }
      } else if (c == '\n') {
        if (p != 4) {
          printstring("Your guess has to be a word of four letters.  Try again.\n");
          goto retype_humans_guess;
        }
        break;
      } else {
        // silently ignore bad character
      }
    }
    for (w = 0; w < words; w++) {
      for (p = 0; p < 4; p++) {
        if (guess[p] != word[w][p]) break;
        if (p==3) goto valid;
      }
    }
    // printf("Sorry, \"%s\" is not a word you can use.  Try another one.\n", guess);
    printstring("Sorry, \""); printstring(guess); printstring("\" is not a word you can use.  Try another one.\n"); 
    continue;
  valid:
    // printf("%s: ", guess);
    printstring(guess); printstring(": "); 
    humans_score = score(saved_word, guess, &bulls, &cows);

    recorded_word[guesses] = w;
    recorded_score[guesses] = humans_score;
    guesses += 1;

    for (p = 0; p < bulls; p++) putchar('+');
    for (p = 0; p < cows; p++) putchar('?');
    if (humans_score == 0) printstring("Washout!  Zip!  Nada!");
    putchar('\n');
    if (bulls == 4) {
      printstring("You win!\n");
      dump_marks(guesses);
      goto restart;
    }
    if (guesses == 12) {
      // printf("That's your last guess.  I win.  My word was '%s'\n", saved_word);
      printstring("That's your last guess.  I win.  My word was '"); printstring(saved_word); printstring("'\n"); 
      dump_marks(guesses);
      goto restart;
    }
  }

  exit(0);  // re-enter S-BUG
  return 0;
}
